# -*- coding: utf-8 -*-
r"""
Random partial injections and Stallings graphs

EXAMPLES::

    sage: from slabbe import number_of_partial_injection
    sage: number_of_partial_injection(10)
    [1,
    100,
    4050,
    86400,
    1058400,
    7620480,
    31752000,
    72576000,
    81648000,
    36288000,
    3628800]

Random partial injections on ``[0, 1, ..., 6]``::

    sage: from slabbe import random_partial_injection
    sage: random_partial_injection(7)
    [None, None, 1, 3, None, 0, None]
    sage: random_partial_injection(7)
    [5, 1, 0, 3, None, 4, None]

Random Stallings graph on ``[0, 1, ..., 19]`` over 2 letters::

    sage: from slabbe import random_cyclically_reduced_stallings_graph
    sage: G,_,_ = random_cyclically_reduced_stallings_graph(20, 2)
    sage: G
    Looped multi-digraph on 20 vertices

Visualisation of the graph::

    sage: from slabbe import TikzPicture
    sage: tikz = TikzPicture.from_graph(G)      # optional dot2tex
    sage: path_to_file = tikz.pdf()    # not tested

REFERENCES:

- Bassino, Frédérique; Nicaud, Cyril; Weil, Pascal Random generation of
  finitely generated subgroups of a free group.  Internat. J. Algebra
  Comput. 18 (2008), no. 2, 375–405. 

"""
#*****************************************************************************
#       Copyright (C) 2017 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from sage.rings.integer_ring import ZZ
from sage.probability.probability_distribution import GeneralDiscreteDistribution
from sage.misc.prandom import shuffle, sample
from sage.graphs.digraph import DiGraph
from sage.functions.other import binomial, factorial

def number_of_partial_injection(n, algorithm='binomial'):
    r"""
    Return the number of partial injections on an set of `n` elements
    defined on a subset of `k` elements for each `k` in `0, 1, ..., n`.

    INPUT:

    - ``n`` -- integer
    - ``algorithm`` -- string (default: ``'binomial'``), ``'binomial'``
      or ``'recursive'``. When n>50, the binomial coefficient approach is
      faster (linear time vs quadratic time).

    OUTPUT:

        list

    .. NOTE::

        The recursive code of this function was originally written by
        Vincent Delecroix (Nov 30, 2017) the day after a discussion with
        Pascal Weil and me at LaBRI.

    EXAMPLES::

        sage: from slabbe import number_of_partial_injection
        sage: number_of_partial_injection(0)
        [1]
        sage: number_of_partial_injection(1)
        [1, 1]
        sage: number_of_partial_injection(2)
        [1, 4, 2]
        sage: number_of_partial_injection(3)
        [1, 9, 18, 6]
        sage: number_of_partial_injection(4)
        [1, 16, 72, 96, 24]
        sage: number_of_partial_injection(5)
        [1, 25, 200, 600, 600, 120]
        sage: number_of_partial_injection(6)
        [1, 36, 450, 2400, 5400, 4320, 720]
        sage: number_of_partial_injection(7)
        [1, 49, 882, 7350, 29400, 52920, 35280, 5040]
        sage: number_of_partial_injection(8)
        [1, 64, 1568, 18816, 117600, 376320, 564480, 322560, 40320]

    TESTS::

        sage: number_of_partial_injection(8, algorithm='recursive')
        [1, 64, 1568, 18816, 117600, 376320, 564480, 322560, 40320]

    REFERENCE:

        https://oeis.org/A144084
    """
    if algorithm == 'binomial':
        return [binomial(n,k)**2*factorial(k) for k in range(n+1)]
    elif algorithm == 'recursive':
        L = [ZZ(1)]
        for t in range(1, n+1):
            L.append(t * L[-1])
            for k in range(t-1, 0, -1):
                L[k] = (t * L[k]) // (t-k) + t * L[k-1]
        return L

def random_partial_injection(n):
    r"""
    Return a uniformly chosen random partial injection on 0, 1, ..., n-1.

    INPUT:

    - ``n`` -- integer

    OUTPUT:

        list

    EXAMPLES::

        sage: from slabbe import random_partial_injection
        sage: random_partial_injection(10)
        [3, 5, 2, None, 1, None, 0, 8, 7, 6]
        sage: random_partial_injection(10)
        [1, 7, 4, 8, 3, 5, 9, None, 6, None]
        sage: random_partial_injection(10)
        [5, 6, 8, None, 7, 4, 0, 9, None, None]

    TODO::

        Adapt the code once this is merged:

        https://trac.sagemath.org/ticket/24416

    AUTHORS:

    - Sébastien Labbé and Vincent Delecroix, Nov 30, 2017, Sage Thursdays
      at LaBRI
    """
    L = number_of_partial_injection(n)
    s = sum(L).n()
    L = [a/s for a in L]   # because of the bug #24416
    X = GeneralDiscreteDistribution(L)
    k = X.get_random_element()
    codomain = sample(range(n), k)
    missing = [None]*(n-k)
    codomain.extend(missing)
    shuffle(codomain)
    return codomain

def random_cyclically_reduced_stallings_graph(n, r=2, verbose=False, merge=False):
    r"""
    Return a uniformly chosen Stallings graph of n vertices over r letters.

    INPUT:

    - ``n`` -- integer, size of graph
    - ``r`` -- integer (default: ``2``), number of generators of the free
      group
    - ``verbose`` -- bool (default: ``False``)

    .. NOTE::

        The probability that G is connected is 1 - 2^r / n^(r-1) +
        o(1/n^(r-1)) which is approx. 1

    OUTPUT:

        digraph, integer, integer

    EXAMPLES::

        sage: from slabbe import random_cyclically_reduced_stallings_graph
        sage: G,_,_ = random_cyclically_reduced_stallings_graph(20, 2)
        sage: G
        Looped multi-digraph on 20 vertices

    ::

        sage: random_cyclically_reduced_stallings_graph(20, 5)[0]
        Looped multi-digraph on 20 vertices

    With verbose output::

        sage: G = random_cyclically_reduced_stallings_graph(20, 2, verbose=True)   # random
        rejecting because graph is not connected
        rejecting because graph has a vertex of degree <=1
        rejecting because graph has a vertex of degree <=1
        rejecting because graph has a vertex of degree <=1

    For displaying purposes, the following merges the multiedges
    automatically::

        sage: G,_,_ = random_cyclically_reduced_stallings_graph(20, 2)
        sage: from slabbe import TikzPicture
        sage: tikz = TikzPicture.from_graph(G)    # optional dot2tex
        sage: _ = tikz.pdf(view=False)            # optional dot2tex

    AUTHORS:

    - Sébastien Labbé and Pascal Weil, Dec 14, 2017, Sage Thursdays at LaBRI
    """
    from sage.misc.latex import LatexExpr

    # reject statistics
    not_connected_count = 0
    has_degree_1_count = 0

    while True:
        injections = [random_partial_injection(n) for _ in range(r)]

        edges = []
        for i,injection in enumerate(injections):
            label = LatexExpr('a_{}'.format(i))
            edges.extend([(j,image_j,label) for (j,image_j) in enumerate(injection)
                                        if not image_j is None])

        G = DiGraph([range(n), edges], format='vertices_and_edges',
                loops=True, multiedges=True)

        if not G.is_connected():
            not_connected_count += 1
            if verbose:
                print("rejecting because graph is not connected")
            continue

        if not all(d>=2 for d in G.degree_iterator()):
            has_degree_1_count += 1
            if verbose:
                print("rejecting because graph has a vertex of degree <=1")
            continue

        return G, not_connected_count, has_degree_1_count

def reject_statistics(n, r=2, sample_size=50, verbose=False):
    r"""
    Return return reject statistics when randomly chosing Stallings graph
    of n vertices over r letters.

    INPUT:

    - ``n`` -- integer, size of graph
    - ``r`` -- integer (default: ``2``), number of generators of the free
      group
    - ``n`` -- integer (default: ``50``), size of sample
    - ``verbose`` -- bool (default: ``False``)

    OUTPUT:

        histogram

    EXAMPLES::

        sage: from slabbe.partial_injection import reject_statistics
        sage: h = reject_statistics(50, verbose=True)   # random
        not connected: Counter({0: 48, 1: 2})
        has degree 1: Counter({0: 27, 1: 18, 2: 3, 3: 1, 4: 1})
        sage: h.save('h_50.png', title='size of graph=50') # not tested

    ::

        sage: h = reject_statistics(100, verbose=True)  # random
        not connected: Counter({0: 48, 1: 2})
        has degree 1: Counter({0: 41, 1: 8, 2: 1})
        sage: h.save('h_100.png', title='size of graph=100') # not tested

    ::

        sage: h = reject_statistics(500, verbose=True)       # not tested (30s)
        not connected: Counter({2: 5, 4: 5, 5: 5, 0: 4, 1: 4, 8: 4, 3: 3, 16:
            3, 6: 2, 11: 2, 15: 2, 18: 2, 23: 2, 7: 1, 10: 1, 44: 1, 13: 1, 49:
            1, 19: 1, 21: 1})
        has degree 1: Counter({0: 14, 1: 9, 3: 8, 2: 5, 4: 3, 5: 3, 6: 2, 9: 2,
            7: 1, 8: 1, 13: 1, 15: 1})
        sage: h.save('h_500.png', title='size of graph=500') # not tested

    ::

        sage: h = reject_statistics(1000, verbose=True)  # not tested (2min30s)
        not connected: Counter({8: 4, 26: 3, 3: 2, 7: 2, 9: 2, 10: 2, 14: 2,
        15: 2, 17: 2, 18: 2, 27: 2, 40: 2, 59: 2, 0: 1, 1: 1, 4: 1, 5: 1, 11:
        1, 13: 1, 19: 1, 20: 1, 21: 1, 22: 1, 28: 1, 44: 1, 48: 1, 51: 1, 52:
        1, 53: 1, 58: 1, 63: 1, 66: 1, 75: 1, 121: 1})
        has degree 1: Counter({2: 9, 0: 7, 1: 6, 4: 6, 3: 4, 5: 4, 7: 4, 8: 2,
            6: 1, 9: 1, 11: 1, 12: 1, 13: 1, 15: 1, 17: 1, 26: 1})
        sage: h.save('h_1000.png', title='size of graph=1000') # not tested

    """
    from sage.plot.histogram import histogram
    A = [random_cyclically_reduced_stallings_graph(n, r) for _ in range(sample_size)]
    _,s,t = zip(*A)
    if verbose:
        from collections import Counter
        print("not connected:", Counter(s))
        print("has degree 1:", Counter(t))
    h = histogram([s,t])
    return h




# -*- coding: utf-8 -*-
r"""
Discrete Hyperplanes

Intersection of a plane and a tube::

    sage: from slabbe import DiscretePlane, DiscreteTube
    sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
    sage: d = DiscreteTube([-5,5],[-5,5])
    sage: I = p & d
    sage: I
    Intersection of the following objects:
    Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
    DiscreteTube: Preimage of [-5, 5] x [-5, 5] by a 2 by 3 matrix
    sage: len(list(I))
    115

Intersection of a line and a box::

    sage: from slabbe import DiscreteLine, DiscreteBox
    sage: L = DiscreteLine([pi,sqrt(2)], pi+sqrt(2), mu=0)
    sage: b = DiscreteBox([-5,5],[-5,5])
    sage: I = L & b
    sage: I
    Intersection of the following objects:
    Set of points x in ZZ^2 satisfying: 0 <= (pi, sqrt(2)) . x + 0 < pi + sqrt(2)
    [-5, 5] x [-5, 5]


TODO:

    - do some dimension checking for DiscreteLine and DiscretePlane

"""

#*****************************************************************************
#       Copyright (C) 2010-2016 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from sage.rings.real_mpfr import RealField
from sage.functions.other import ceil, sqrt
from sage.modules.free_module_element import vector
from sage.misc.cachefunc import cached_method
from slabbe.discrete_subset import DiscreteSubset
################################################
# Discrete plane and hyperplanes
################################################
class DiscreteHyperplane(DiscreteSubset):
    r"""
    This is the set of point `p` such that

        `0 \leq  p \cdot v - mu < \omega`

    INPUT:

    - ``v`` - normal vector
    - ``omega`` - width
    - ``mu`` - intercept (optional, default: 0)

    EXAMPLES::

        sage: from slabbe import DiscreteLine
        sage: L = DiscreteLine([pi,sqrt(2)], pi+sqrt(2), mu=10)
        sage: L
        Set of points x in ZZ^2 satisfying: 0 <= (pi, sqrt(2)) . x + 10 < pi + sqrt(2)

    ::

        sage: from slabbe import DiscretePlane
        sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
        sage: p
        Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8

    ::

        sage: from slabbe import DiscreteHyperplane
        sage: p = DiscreteHyperplane([1,3,7,9], 20, mu=13)
        sage: p
        Set of points x in ZZ^4 satisfying: 0 <= (1, 3, 7, 9) . x + 13 < 20

    TESTS::

        sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=20)
        sage: vector((0,0,0)) in p
        False
        sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
        sage: vector((0,0,0)) in p
        True

    ::

        sage: p = DiscreteHyperplane((2,3,4,5), 10)
        sage: p.dimension()
        4

    ::

        sage: L = DiscreteLine([1,pi], 1+pi, mu=20)
        sage: vector((0,0)) in L
        False
        sage: L = DiscreteLine([1,pi], 1+pi, mu=0)
        sage: vector((0,0)) in L
        True
    """
    def __init__(self, v, omega, mu=0, prec=None):
        r"""
        EXAMPLES::

            sage: from slabbe import DiscreteHyperplane
            sage: p = DiscreteHyperplane([1,pi,7], 1+pi+7, mu=0)
            sage: p
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8

        ::

            sage: p = DiscreteHyperplane([1,pi,7], 1+pi+7, mu=20)
            sage: vector((0,0,0)) in p
            False
            sage: p = DiscreteHyperplane([1,pi,7], 1+pi+7, mu=0)
            sage: vector((0,0,0)) in p
            True
        """
        if prec is None:
            self._v = vector(v)
            self._omega = omega
            self._mu = mu
        else:
            RF = RealField(prec=prec)
            self._v = vector(RF, v)
            self._omega = RF(omega)
            self._mu = RF(mu)
        def contain(p):
            #print("est-ce proche : ", self._v.dot_product(p) + self._mu)
            return  0 <= self._v.dot_product(p) + self._mu < self._omega
        DiscreteSubset.__init__(self, dimension=len(self._v), predicate=contain)

    @cached_method
    def roots(self):
        r"""
        Return the roots, i.e., a list of elements in self.

        It also makes sure the roots are in self and raises an error otherwise.

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: P = DiscretePlane([3,4,5], 12, mu=20)
            sage: P.roots()
            [(-1, -1, -1)]
            sage: all(p in P for p in P.roots())
            True
        """
        p = self.an_element()
        p = self._space(p)
        p.set_immutable()
        if not p in self: 
            raise ValueError("root element (={}) provided at"
                    " initialisation is not in self".format(p))
        self._roots = [p]
        return self._roots

    def _repr_(self):
        r"""
        EXAMPLES::

            sage: from slabbe import DiscreteHyperplane
            sage: p = DiscreteHyperplane([1,pi,7], 1+pi+7, mu=10)
            sage: p
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 10 < pi + 8
        """
        s = "Set of points x in ZZ^{} satisfying: ".format(self.dimension())
        s +=  "0 <= {} . x + {} < {}".format(self._v,self._mu, self._omega)
        return s

    def an_element(self, x=0, y=0):
        r"""
        Returns an element in self.

        EXAMPLES::

            sage: from slabbe import DiscreteHyperplane
            sage: p = DiscreteHyperplane([1,pi,7], 1+pi+7, mu=10)
            sage: p.an_element()
            (0, 0, 0)

        ::

            sage: from slabbe import DiscreteLine
            sage: L = DiscreteLine([pi,sqrt(2)], pi+sqrt(2), mu=10)
            sage: L.an_element()
            (-2, -2)

        ::

            sage: L = DiscreteLine([pi,sqrt(2)], pi+sqrt(2), mu=0)
            sage: L.an_element()
            (0, 0)

        """
        dimension = len(self._v)
        if dimension == 2:
            return self._an_element_2d(x=x,y=y)
        elif dimension == 3:
            return self._an_element_3d(x=x,y=y)
        else:
            raise NotImplementedError("implemented only for dimension(=%s) 2 or 3" % dimension)

    def _an_element_3d(self, x=0, y=0):
        r"""
        Returns an element in self.

        EXAMPLES::

            sage: from slabbe import DiscreteHyperplane
            sage: p = DiscreteHyperplane([1,pi,7], 1+pi+7, mu=10)
            sage: p._an_element_3d()
            (0, 0, 0)
        """
        a,b,c = self._v
        x_sqrt3 = ceil(x / sqrt(3))
        left  = ((a+b) * y + (a-b) * x_sqrt3 - self._mu) / (a+b+c)
        right = ((a+b) * y + (a-b) * x_sqrt3 - self._mu + self._omega) / (a+b+c)
        #print("left, right = ", left, right)
        #print("left, right = ", ceil(left), ceil(right)-1)
        # left <= z <= right
        znew = ceil(left)
        xnew = znew - y - x_sqrt3
        ynew = znew - y + x_sqrt3
        znew = ceil(right)-1
        #print(xnew, ynew, znew)
        #print(vector((xnew, ynew, znew)) in self)
        #print(vector((x,y,ceil(right)-1)) in self)
        v = vector((xnew, ynew, znew))
        if v in self:
            v.set_immutable()
            return v
        else:
            print("%s not in the plane" % v)
            print("trying similar points")
            v = vector((xnew, ynew, znew-1))
            if v in self:
                v.set_immutable()
                return v
            v = vector((xnew, ynew, znew+1))
            if v in self:
                v.set_immutable()
                return v
        raise ValueError("%s not in the plane" % v)

        # minimum = - floor(self._mu)
        # maximum = ceil(self._omega - self._mu) - 1
        # # print(minimum, maximum)
        # # minimum <= z <= maximum
        # assert vector((0,0,maximum)) in self
        # return vector((0,0,maximum))
    def _an_element_2d(self, x=0, y=0):
        r"""
        Returns an element in self.

        EXAMPLES::

            sage: from slabbe import DiscreteLine
            sage: L = DiscreteLine([pi,sqrt(2)], pi+sqrt(2), mu=10)
            sage: L._an_element_2d()
            (-2, -2)

        """
        assert self.dimension() == 2, "dimension must be 2"
        p = vector((x,y))
        while True:
            if p in self:
                p.set_immutable()
                return p
            elif -p in self:
                mp = -p
                mp.set_immutable()
                return mp
            else:
                p += vector((1,1))


    def level_value(self, p):
        r"""
        Return the level value of a point p.

        INPUT:

        - ``p`` - point in the space

        EXAMPLES::

            sage: from slabbe import DiscreteHyperplane
            sage: H = DiscreteHyperplane([1,3,7,9], 20, mu=13)
            sage: p = H._space((1,2,3,4))
            sage: H.level_value(p)
            64
        """
        return self._v.dot_product(p)

DiscretePlane = DiscreteHyperplane
DiscreteLine = DiscreteHyperplane



# coding=utf-8
r"""
Bond Percolation

This is an implementation of bond percolation. See Chapter 3 of [POG]_. See
also my blog post `Percolation and self-avoiding walks`__ related to this
code.

__ http://www.liafa.univ-paris-diderot.fr/~labbe/blogue/2012/12/percolation-and-self-avoiding-walks/

AUTHORS:

- Sebastien Labbe (2012-12-17): initial version, for `pog`__ lecture group

__ http://www.liafa.univ-paris-diderot.fr/~jolivet/pog/

REFERENCES:

.. [POG] Geoffrey Grimmett, Probability on Graphs,
   http://www.statslab.cam.ac.uk/~grg/books/pgs.html

EXAMPLES:

Bond percolation sample
-----------------------

We construct a bond percolation sample in dimension d=2 with probability of
open edges p=0.3::

    sage: from slabbe import BondPercolationSample
    sage: S = BondPercolationSample(p=0.3, d=2)
    sage: S
    Bond percolation sample d=2 p=0.300

An edge is defined uniquely as a starting point in Z^d and an axis
direction given by an integer i such that 1 <= i <= d. One may ask if a
given edge is in the sample S::

    sage: ((34,56), 2) in S     # random
    False

The result is cached so the same answer is returned again::

    sage: ((34,56), 2) in S     # random
    False
    sage: ((34,56), 2) in S     # random
    False

The cluster containing the point zero is returned as an iterator::

    sage: S.cluster()
    <generator object at ...>

It may be finite of infinite. If you believe it is finite, you may compute
its cardinality. If the cluster is infinite, it will not halt::

    sage: S.cluster_cardinality() # not tested, might not halt
    13

For larger values of p, the cluster might be larger if not infinite. In
this case you may want to stop the computation at a certain point
determined in advance. The following method does this. And it returns the
cardinality if it is smaller than the stop value::

    sage: S = BondPercolationSample(p=0.45, d=2)
    sage: S.cluster_cardinality_stop_at(stop=10)          # random
    '>=10'
    sage: S.cluster_cardinality_stop_at(stop=100)          # random
    '>=100'
    sage: S.cluster_cardinality_stop_at(stop=1000)          # random
    625

Bond percolation samples
------------------------

Construction of 20 bond percolation samples. For each of them, compute the
cardinality of the open cluster containing zero::

    sage: from slabbe import BondPercolationSamples
    sage: S20 = BondPercolationSamples(p=0.4, d=2, n=20)
    sage: S20.cluster_cardinality(stop=100)               # random
    [4, 2, 1, 4, 1, 10, 62, 71, 1, 25, 19, 2, 2, 42, '>=100', 1, 18, 2, '>=100', 20]

By considering "larger than 100" to be an infinite cluster, this gives a
value of 2/20 = 0.10 for the percolation probability::

    sage: S20.percolation_probability(stop=100)        # random
    0.100

By increasing the stop value, the computations can be redone again *on the
same samples*. In this case, by using a stop value of 1000, we get a value
of 0 for the percolation probability of p=0.4::

    sage: S20.cluster_cardinality(stop=1000)             # random
    [4, 2, 1, 4, 1, 10, 62, 71, 1, 25, 19, 2, 2, 42, 176, 1, 18, 2, 186, 20]
    sage: S20.percolation_probability(stop=1000)         # random
    0.000

Percolation probability
-----------------------

One can define the percolation probability function for a given dimension
d. It will generate n samples and consider the cluster to be infinite if
its cardinality is larger than the given stop value::

    sage: from slabbe import PercolationProbability
    sage: T = PercolationProbability(d=2, n=10, stop=100)
    sage: T
    Percolation Probability $\theta(p)$
    d = dimension = 2
    n = # samples = 10
    stop counting at = 100

Compute the value for a certain probability p::

    sage: T(0.4534)             # random
    0.300

Of course, this value will change for another equal percolation
probability since it depends on the samples::

    sage: T = PercolationProbability(d=2, n=10, stop=100)
    sage: T(0.4534)              # random
    0.600

Anyway, it is useful to draw the plot of the percolation probability::

    sage: T = PercolationProbability(d=2, n=10, stop=100)
    sage: T.return_plot((0,1), adaptive_recursion=4, plot_points=4)     # optional long
    Graphics object consisting of 2 graphics primitives

Here we use Sage adaptative recursion algorithm for drawing plots which
finds the particular important intervals to ask for more values of the
function. See help section of plot function for details. Because T might be
long to compute we start with only 4 points

TODO
----

 - Make it 100% doctested (presently 21/24 = 87%)
 - Base it on DiscreteSubset code
 - Fix tikz2pdf use

Do we want to use?::

    sage: from sage.sets.set_from_iterator import EnumeratedSetFromIterator
    sage: from itertools import count
    sage: S = EnumeratedSetFromIterator(count)
    sage: S
    {0, 1, 2, 3, 4, ...}

and ?::

    sage: M = FiniteSetMaps(["a", "b"], [3, 4, 5]); M
    Maps from {'a', 'b'} to {3, 4, 5}

Methods and classes
-------------------

"""
#*****************************************************************************
#       Copyright (C) 2012 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import os
import itertools
from sage.structure.sage_object import SageObject
from sage.combinat.backtrack import TransitiveIdeal
from sage.rings.infinity import Infinity
from sage.graphs.graph import Graph
from sage.functions.generalized import sgn
from sage.functions.other import abs
from sage.misc.functional import numerical_approx
from sage.misc.cachefunc import cached_method
from sage.misc.latex import LatexExpr
from sage.plot.graphics import Graphics
from sage.plot.point import point
from sage.plot.circle import circle
from sage.plot.line import line
from sage.plot.text import text
from sage.plot.plot import graphics_array, plot


class BondPercolationSample(SageObject):
    r"""
    Let $L^d = (Z^d,E^d)$ be the hypercubic lattice.

    A sample contained in the set {0,1}^{E^d}.

    An edge e in E is open (=1) in the sample with probability p.

    Cached __contains__ method below does the job of memory.
    """
    def __init__(self, p, d=2):
        r"""
        INPUT:

        - ``p`` - real number in [0,1]
        """
        self._p = p
        self._dimension = d

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.4,2)
            sage: S
            Bond percolation sample d=2 p=0.400
        """
        s = "Bond percolation sample d=%s p=%.3f" % (self._dimension, self._p)
        return s

    @cached_method
    def __contains__(self, arg):
        r"""
        Return True with probability p.

        INPUT:

        - ``arg`` - a 2-tuple containing:

          - ``pt`` - point in Z^d
          - ``direction`` - integer, possible values are 1, 2, ..., d and -1,
            -2, ..., -d.

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5)
            sage: (S.zero(), 1) in S        # random
            True
            sage: (S.zero(), -1) in S        # random
            True
        """
        pt, direction = arg
        # here because creates docbuild error when the import is global
        from random import random 
        return random() < self._p

    def neighbor(self, pt, d):
        r"""
        Return the neighbors of the point pt in direction d.

        INPUT:

        - ``pt`` - tuple, point in Z^d
        - ``direction`` - integer, possible values are 1, 2, ..., d and -1,
          -2, ..., -d.

        EXAMPLES:

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5,2)
            sage: S.neighbor((2,3),1)
            (3, 3)
            sage: S.neighbor((2,3),2)
            (2, 4)
            sage: S.neighbor((2,3),-1)
            (1, 3)
            sage: S.neighbor((2,3),-2)
            (2, 2)

        """
        a = sgn(d)
        d = abs(d)
        return tuple(pt[k]+a if k==d-1 else pt[k] for k in range(self._dimension))

    def children(self, pt):
        r"""
        Return an iterator over open neighbors of the point pt.

        INPUT:

        - ``pt`` - tuple, point in Z^d
        - ``m`` - integer, limit

        EXAMPLES:

        The result is consistent::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5)
            sage: list(S.children((0,0)))        # random
            [(1, 0), (-1, 0), (0, -1)]

        Might be different for another sample::

            sage: S = BondPercolationSample(0.5)
            sage: list(S.children((0,0)))        # random
            [(-1, 0)]

        In dimension 3::

            sage: S = BondPercolationSample(0.5,3)
            sage: list(S.children((0,0,0)))        # random
            [(1, 0, 0), (0, -1, 0), (0, 0, -1)]
            sage: S = BondPercolationSample(0.5,3)
            sage: list(S.children((0,0,0)))        # random
            [(1, 0, 0), (-1, 0, 0)]
            sage: list(S.children((0,0,0)))        # random
            [(1, 0, 0), (-1, 0, 0)]

        ::

            sage: S = BondPercolationSample(1,2)
            sage: list(S.children((0,0)))        # random
            [(1, 0), (-1, 0), (0, 1), (0, -1)]
            sage: S = BondPercolationSample(1,3)
            sage: list(S.children((0,0,0)))        # random
            [(1, 0, 0), (-1, 0, 0), (0, 1, 0), (0, -1, 0), (0, 0, 1), (0, 0, -1)]
        """
        for d in range(1, self._dimension+1):
            if (pt, d) in self:
                yield self.neighbor(pt, d)
            opposite_pt = self.neighbor(pt, -d)
            if (opposite_pt, d) in self:
                yield opposite_pt

    def zero(self):
        r"""
        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5,3)
            sage: S.zero()
            (0, 0, 0)

        ::

            sage: S = BondPercolationSample(0.5,5)
            sage: S.zero()
            (0, 0, 0, 0, 0)
        """
        return (0,) * self._dimension

    def cluster(self, pt=None):
        r"""
        Return an iterator over the open cluster containing the point pt.

        INPUT:

        - ``pt`` - tuple, point in Z^d. If None, pt=zero is considered.

        EXAMPLES:

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5)
            sage: it = S.cluster()
            sage: next(it)
            (0, 0)

        """
        if pt is None:
            pt = self.zero()
        generators = [pt]
        return iter(TransitiveIdeal(self.children, generators))

    def cluster_cardinality(self, pt=None):
        r"""
        INPUT:

        - ``pt`` - tuple, point in Z^d. If None, pt=zero is considered.

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: BondPercolationSample(0.01).cluster_cardinality() # random
            1
            sage: BondPercolationSample(0.4).cluster_cardinality()  # random
            28
        """
        return sum(1 for _ in self.cluster(pt=pt))

    def cluster_cardinality_stop_at(self, stop, pt=None):
        r"""
        Return the cardinality of the cluster or the strin ">=STOP" if the size is
        larger than stop value.

        INPUT:

        - ``stop`` - integer
        - ``pt`` - tuple, point in Z^d. If None, pt=zero is considered.

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.3,2)
            sage: S.cluster_cardinality()                # random
            13
            sage: S.cluster_cardinality_stop_at(1000)    # random
            13
            sage: S.cluster_cardinality_stop_at(100)     # random
            13
            sage: S.cluster_cardinality_stop_at(10)      # random
            '>=10'
        """
        it = itertools.islice(self.cluster(pt=pt), stop)
        size = sum(1 for _ in it)
        if size == stop:
            return ">=%s" % stop
        else:
            return size

    def edges_in_box(self, m):
        r"""
        Return an iterator over all edges in the primal box [-m,m]^d.

        INPUT:

        - ``m`` - integer

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(1,2)
            sage: for a in S.edges_in_box(1): print(a)
            ((-1, -1), (0, -1))
            ((-1, -1), (-1, 0))
            ((-1, 0), (0, 0))
            ((-1, 0), (-1, 1))
            ((0, -1), (1, -1))
            ((0, -1), (0, 0))
            ((0, 0), (1, 0))
            ((0, 0), (0, 1))
        """
        R = range(-m, m)
        L = [R]*self._dimension
        positive_directions = range(1, self._dimension+1)
        for pt in itertools.product(*L):
            for d in positive_directions:
                if (pt,d) in self:
                    neighbor = self.neighbor(pt,d)
                    yield (pt, neighbor)

    def cluster_in_box(self, m, pt=None):
        r"""
        Return the cluster (as a list) in the primal box [-m,m]^d
        containing the point pt.

        INPUT:

        - ``m`` - integer
        - ``pt`` - tuple, point in Z^d. If None, pt=zero is considered.

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.3,2)
            sage: S.cluster_in_box(2)         # random
            [(-2, -2), (-2, -1), (-1, -2), (-1, -1), (-1, 0), (0, 0)]
        """
        G = Graph()
        G.add_edges(self.edges_in_box(m))
        if pt is None:
            pt = self.zero()
        if pt in G:
            return G.connected_component_containing_vertex(pt)
        else:
            return []

    def plot(self, m, pointsize=100, thickness=3, axes=False):
        r"""
        Return 2d graphics object contained in the primal box [-m,m]^d.

        INPUT:

        - ``pointsize``, integer (default:``100``),
        - ``thickness``, integer (default:``3``),
        - ``axes``, bool (default:``False``),

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5,2)
            sage: S.plot(2)           # optional long

        It works in 3d!!::

            sage: S = BondPercolationSample(0.5,3)
            sage: S.plot(3, pointsize=10, thickness=1)     # optional long
            Graphics3d Object

        """
        s = ""
        s += "\\begin{tikzpicture}\n"
        s += "[inner sep=0pt,thick,\n"
        s += "reddot/.style={fill=red,draw=red,circle,minimum size=5pt}]\n"
        s += "\\clip %s rectangle %s;\n" % ((-m-.4,-m-.4), (m+.4,m+.4))
        G = Graphics()
        for u in self.cluster_in_box(m+1):
            G += point(u, color='blue', size=pointsize)
        for (u,v) in self.edges_in_box(m+1):
            G += line((u,v), thickness=thickness, alpha=0.8)
        G += text("p=%.3f" % self._p, (0.5,1.03), axis_coords=True, color='black')
        G += circle((0,0), 0.5, color='red', thickness=thickness)
        if self._dimension == 2:
            G.axes(axes)
        return G

    def tikz(self, m):
        r"""
        Return tikz code.

        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: S = BondPercolationSample(0.5,2)
            sage: S.tikz(2)
            \begin{tikzpicture}
            [inner sep=0pt,thick, 
             reddot/.style={fill=red,draw=red,circle,minimum size=5pt}]
            \clip (-2.4, -2.4) rectangle (2.4, 2.4);
            \draw (..., ...) -- (..., ...);
            ...
            \node[circle,fill=none,draw=red,minimum size=0.8cm,ultra thick,inner sep=0pt] at (0,0) {};
            \node[above right] at (0,0) {$(0, 0)$};
            \end{tikzpicture}

        """
        s = ""
        s += "\\begin{tikzpicture}\n"
        s += "[inner sep=0pt,thick,\n"
        s += "reddot/.style={fill=red,draw=red,circle,minimum size=5pt}]\n"
        s += "\\clip %s rectangle %s;\n" % ((-m-.4,-m-.4), (m+.4,m+.4))
        for (u,v) in self.edges_in_box(m+1):
            s += "\\draw %s -- %s;\n" % (u,v)
        for u in self.cluster_in_box(m+1):
            s += "\\node[reddot] at %s {};\n" % (u,)
        s += "\\node[circle,fill=none,draw=red,minimum size=0.8cm,ultra thick,inner sep=0pt] at (0,0) {};\n"
        s += "\\node[above right] at (0,0) {$%s$};\n" % (self.zero(),)
        s += "\\end{tikzpicture}\n"
        return LatexExpr(s)

    def save_pdf(self, m):
        r"""
        EXAMPLES::

            sage: from slabbe import BondPercolationSample
            sage: BondPercolationSample(0.3, d=2).save_pdf(20)    # optional long
            Creation du fichier tikz_sample_d2_p300_m20.tikz
            Using template '/Users/slabbe/.tikz2pdf.tex'.
            tikz2pdf: calling pdflatex...
            tikz2pdf: Output written to 'tikz_sample_d2_p300_m20.pdf'.
        """
        tikz = self.tikz(m)
        prefix = "tikz_sample_d%s_p%3d_m%s" % (self._dimension,1000*self._p, m)
        prefix = prefix.replace(' ','0')
        filename = prefix + '.tikz'
        with open(filename, 'w') as f:
            f.write(tikz)
            print("Creation du fichier %s" % filename)
        os.system("tikz2pdf %s" % filename)
        #os.system("convert %s.pdf %s.png" % (prefix, prefix))

class BondPercolationSamples(SageObject):
    r"""
    Return a list of n BondPercolationSample of given parameter p and
    dimension d.

    EXAMPLES::

        sage: from slabbe import BondPercolationSamples
        sage: BondPercolationSamples(0.2,2,3)
        <slabbe.bond_percolation.BondPercolationSamples object at ...>
    """
    def __init__(self, p, d, n):
        r"""
        """
        self._p = p
        self._dimension = d
        self._n = n
        self._list = [BondPercolationSample(p,d) for _ in range(n)]

    @cached_method
    def cluster_cardinality(self, stop):
        r"""
        Return the list of cardinality of the cluster for each sample
        or +Infinity if the size is larger than stop value.

        INPUT:

        - ``stop`` - integer

        EXAMPLES::

            sage: from slabbe import BondPercolationSamples
            sage: S20 = BondPercolationSamples(p=0.2, d=2, n=20)
            sage: S20.cluster_cardinality(100)       # random
            [1, 4, 1, 2, 2, 6, 5, 1, 5, 9, 2, 2, 2, 1, 2, 4, 4, 3, 2, 1]

        ::

            sage: d = 2
            sage: n = 5
            sage: for p in srange(0,1,0.1): print(p,BondPercolationSamples(p,d,n).cluster_cardinality(100)) # optional long
            0.000000000000000 [1, 1, 1, 1, 1]
            0.100000000000000 [5, 1, 2, 1, 2]
            0.200000000000000 [3, 1, 4, 1, 1]
            0.300000000000000 [11, 7, 2, 1, 3]
            0.400000000000000 [3, 3, 35, 10, '>=100']
            0.500000000000000 ['>=100', '>=100', '>=100', '>=100', 26]
            0.600000000000000 ['>=100', '>=100', '>=100', '>=100', '>=100']
            0.700000000000000 ['>=100', '>=100', '>=100', '>=100', '>=100']
            0.800000000000000 ['>=100', '>=100', '>=100', '>=100', '>=100']
            0.900000000000000 ['>=100', '>=100', '>=100', '>=100', '>=100']

        """
        return [S.cluster_cardinality_stop_at(stop) for S in self._list]

    def ntimes_over_size(self, stop):
        r"""
        EXAMPLES::

            sage: from slabbe import BondPercolationSamples
            sage: S = BondPercolationSamples(0.2,2,20)
            sage: S.ntimes_over_size(100)    # random
            0
            sage: S = BondPercolationSamples(0.4,2,20)
            sage: S.ntimes_over_size(100)    # random
            1
            sage: S = BondPercolationSamples(0.5,2,20)
            sage: S.ntimes_over_size(100)    # random
            17
        """
        L = self.cluster_cardinality(stop)
        return sum(1 for i in L if isinstance(i, str))

    def percolation_probability(self, stop):
        return numerical_approx(self.ntimes_over_size(stop) / self._n, digits=3)

def percolation_graphics_array(range_p, d, m, ncols=3):
    r"""
    EXAMPLES::

        sage: from slabbe.bond_percolation import percolation_graphics_array
        sage: percolation_graphics_array(srange(0.1,1,0.1), d=2, m=5)    # optional long
        sage: P = percolation_graphics_array(srange(0.45,0.55,0.01), d=2, m=5)  # optional long
        sage: P.save('array_p45_p55_m5.png')     # not tested
        sage: P = percolation_graphics_array(srange(0.45,0.55,0.01), d=2, m=10) # optional long
        sage: P.save('array_p45_p55_m10.png')    # not tested
    """
    pointsize=20
    thickness=1
    L = [BondPercolationSample(p,d).plot(m,pointsize=pointsize,thickness=thickness) for p in range_p]
    nrows = (len(range_p)-1) // ncols + 1
    return graphics_array(L, n=nrows, m=ncols)

def compute_percolation_probability(range_p, d, n, stop):
    r"""
    EXAMPLES::

        sage: from slabbe.bond_percolation import compute_percolation_probability
        sage: compute_percolation_probability(srange(0,0.8,0.1), d=2, n=5, stop=100) # random
        d = 2, n = number of samples = 5
        stop counting at = 100
        p=0.0000, Theta=0.000, if |C|< 100 then max|C|=1
        p=0.1000, Theta=0.000, if |C|< 100 then max|C|=1
        p=0.2000, Theta=0.000, if |C|< 100 then max|C|=5
        p=0.3000, Theta=0.000, if |C|< 100 then max|C|=6
        p=0.4000, Theta=0.000, if |C|< 100 then max|C|=31
        p=0.5000, Theta=1.00, if |C|< 100 then max|C|=-Infinity
        p=0.6000, Theta=1.00, if |C|< 100 then max|C|=-Infinity
        p=0.7000, Theta=1.00, if |C|< 100 then max|C|=-Infinity

    ::

        sage: range_p = srange(0,0.8,0.1)
        sage: compute_percolation_probability(range_p, d=2, n=5, stop=100) # not tested
        d = 2, n = number of samples = 5
        stop counting at = 100
        p=0.0000, Theta=0.000, if |C|< 100 then max|C|=1
        p=0.1000, Theta=0.000, if |C|< 100 then max|C|=1
        p=0.2000, Theta=0.000, if |C|< 100 then max|C|=5
        p=0.3000, Theta=0.000, if |C|< 100 then max|C|=6
        p=0.4000, Theta=0.000, if |C|< 100 then max|C|=31
        p=0.5000, Theta=1.00, if |C|< 100 then max|C|=-Infinity
        p=0.6000, Theta=1.00, if |C|< 100 then max|C|=-Infinity
        p=0.7000, Theta=1.00, if |C|< 100 then max|C|=-Infinity

    ::

        sage: range_p = srange(0.45,0.55,0.01)
        sage: compute_percolation_probability(range_p, d=2, n=10, stop=1000) # not tested
        d = 2, n = number of samples = 10
        stop counting at = 1000
        p=0.4500, Theta=0.000, if |C|< 1000 then max|C|=378
        p=0.4600, Theta=0.000, if |C|< 1000 then max|C|=475
        p=0.4700, Theta=0.000, if |C|< 1000 then max|C|=514
        p=0.4800, Theta=0.100, if |C|< 1000 then max|C|=655
        p=0.4900, Theta=0.700, if |C|< 1000 then max|C|=274
        p=0.5000, Theta=0.700, if |C|< 1000 then max|C|=975
        p=0.5100, Theta=0.700, if |C|< 1000 then max|C|=16
        p=0.5200, Theta=0.700, if |C|< 1000 then max|C|=125
        p=0.5300, Theta=0.900, if |C|< 1000 then max|C|=4
        p=0.5400, Theta=0.700, if |C|< 1000 then max|C|=6

    ::

        sage: range_p = srange(0.475,0.485,0.001)
        sage: compute_percolation_probability(range_p, d=2, n=10, stop=1000) # not tested
        d = 2, n = number of samples = 10
        stop counting at = 1000
        p=0.4750, Theta=0.200, if |C|< 1000 then max|C|=718
        p=0.4760, Theta=0.200, if |C|< 1000 then max|C|=844
        p=0.4770, Theta=0.200, if |C|< 1000 then max|C|=566
        p=0.4780, Theta=0.500, if |C|< 1000 then max|C|=257
        p=0.4790, Theta=0.200, if |C|< 1000 then max|C|=566
        p=0.4800, Theta=0.300, if |C|< 1000 then max|C|=544
        p=0.4810, Theta=0.300, if |C|< 1000 then max|C|=778
        p=0.4820, Theta=0.500, if |C|< 1000 then max|C|=983
        p=0.4830, Theta=0.300, if |C|< 1000 then max|C|=473
        p=0.4840, Theta=0.500, if |C|< 1000 then max|C|=411

    ::

        sage: range_p = srange(0.47,0.48,0.001)
        sage: compute_percolation_probability(range_p, d=2, n=20, stop=2000)  # not tested
        d = 2, n = number of samples = 20
        stop counting at = 2000
        p=0.4700, Theta=0.0500, if |C|< 2000 then max|C|=1666
        p=0.4710, Theta=0.100, if |C|< 2000 then max|C|=1665
        p=0.4720, Theta=0.000, if |C|< 2000 then max|C|=1798
        p=0.4730, Theta=0.0500, if |C|< 2000 then max|C|=1717
        p=0.4740, Theta=0.150, if |C|< 2000 then max|C|=1924
        p=0.4750, Theta=0.150, if |C|< 2000 then max|C|=1893
        p=0.4760, Theta=0.150, if |C|< 2000 then max|C|=1458
        p=0.4770, Theta=0.150, if |C|< 2000 then max|C|=1573
        p=0.4780, Theta=0.200, if |C|< 2000 then max|C|=1762
        p=0.4790, Theta=0.250, if |C|< 2000 then max|C|=951
    """
    print("d = %s, n = number of samples = %s" % (d, n))
    print("stop counting at = %s" % stop)
    for p in range_p:
        p = numerical_approx(p, digits=4)
        S = BondPercolationSamples(p,d,n)
        L = [a for a in S.cluster_cardinality(stop) if not isinstance(a, str)]
        Y = max(L) if L else -Infinity
        theta = S.percolation_probability(stop)
        print("p=%s, Theta=%s, if |C|< %s then max|C|=%s" % (p, theta, stop, Y))


class PercolationProbability(SageObject):
    def __init__(self, d, n, stop, verbose=False):
        r"""
        EXAMPLES::

            sage: from slabbe import PercolationProbability
            sage: f = PercolationProbability(d=2, n=10, stop=100)
            sage: f
            Percolation Probability $\theta(p)$
            d = dimension = 2
            n = # samples = 10
            stop counting at = 100
        """
        self._dimension = d
        self._n = n
        self._stop = stop
        self._verbose = verbose

    def __repr__(self):
        r"""
        EXAMPLES::

            sage: from slabbe import PercolationProbability
            sage: f = PercolationProbability(d=2, n=10, stop=100)
            sage: f
            Percolation Probability $\theta(p)$
            d = dimension = 2
            n = # samples = 10
            stop counting at = 100
        """
        s = "Percolation Probability $\\theta(p)$\n"
        s += "d = dimension = %s\n" % self._dimension
        s += "n = # samples = %s\n" % self._n
        s += "stop counting at = %s" % self._stop
        return s

    @cached_method
    def __call__(self, p ):
        r"""
        EXAMPLES::

            sage: from slabbe import PercolationProbability
            sage: f = PercolationProbability(d=2, n=10, stop=100)
            sage: f
            Percolation Probability $\theta(p)$
            d = dimension = 2
            n = # samples = 10
            stop counting at = 100
            sage: f(0.4534)         # random
            0.300
        """
        if self._verbose:
            print(p)
        S = BondPercolationSamples(p, self._dimension, self._n)
        theta = S.percolation_probability(self._stop)
        return theta

    def return_plot(self, interval=(0,1), adaptive_recursion=4,
            plot_points=4,
            adaptive_tolerance=0.10):
        r"""
        Return a plot of percolation probability using basic sage plot settings.

        INPUT:

        - ``interval``, default=(0,1)
        - ``adaptive_recursion``, default=0
        - ``plot_points``, default=10
        - ``adaptive_tolerance`` default=0.10

        EXAMPLES::

            sage: from slabbe import PercolationProbability
            sage: T = PercolationProbability(d=2, n=10, stop=100)
            sage: T.return_plot()           # optional long
            Graphics object consisting of 1 graphics primitive
        """
        P = plot(self, interval, adaptive_recursion=adaptive_recursion,
                 plot_points=plot_points,
                 adaptive_tolerance=adaptive_tolerance)
        P += text(repr(self), (0.8,0.2))
        return P





# -*- coding: utf-8 -*-
r"""
Infinite words

Methods that are not in Sage (for now!)

AUTHORS:

- Sébastien Labbé, 2016

EXAMPLES::

    sage: from slabbe.infinite_word import derived_sequence
    sage: w = words.ThueMorseWord()
    sage: derived_sequence(w, w[:1])
    word: 0120210121020120210201210120210121020121...
"""
#*****************************************************************************
#       Copyright (C) 2016 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
from sage.combinat.words.words import InfiniteWords
from sage.rings.semirings.non_negative_integer_semiring import NN

def derived_sequence(self, u, coding=False):
    r"""
    Return the derived sequence of according to the return words to a factor of
    self.

    INPUT:

    - ``u`` -- finite word, the length of the nonempty prefix
    - ``coding`` -- boolean (default: ``False``), whether to
      include the return word coding dictionnary

    EXAMPLES::

        sage: from slabbe.infinite_word import derived_sequence
        sage: w = words.ThueMorseWord()
        sage: derived_sequence(w, w[:1])
        word: 0120210121020120210201210120210121020121...
        sage: derived_sequence(w, w[:2])
        word: 0123013201232013012301320130123201230132...
        sage: derived_sequence(w, w[:3])
        word: 0123013201232013012301320130123201230132...

    With the return word coding::

        sage: w = words.ThueMorseWord()
        sage: derived, D = derived_sequence(w, w[:1], True)
        sage: derived
        word: 0120210121020120210201210120210121020121...
        sage: D
        {word: 0: 2, word: 01: 1, word: 011: 0}

    It gets into a cycle of length 1::

        sage: words.ThueMorseWord()
        word: 0110100110010110100101100110100110010110...
        sage: derived_sequence(_, _[:1])
        word: 0120210121020120210201210120210121020121...
        sage: derived_sequence(_, _[:1])
        word: 0123013201232013012301320130123201230132...
        sage: derived_sequence(_, _[:1])
        word: 0123013201232013012301320130123201230132...

    .. NOTE::

        Note that method ``return_words_derivate`` of finite words in Sage does
        the same for finite words but without returning the translation
        dictionnary::

            sage: w = words.ThueMorseWord()
            sage: prefix = w[:1000]
            sage: prefix.return_words_derivate(prefix[:1])
            word: 1231321232131231321312321231321232131232...
    """
    D = {}
    it = (D.setdefault(w, len(D)) for w in self.return_words_iterator(u))
    W = InfiniteWords(alphabet=NN)
    w = W(it, datatype='iter')
    if coding:
        return w, D
    else:
        return w




# -*- coding: utf-8 -*-
r"""
Random generation of Jeandel-Rao tilings.

EXAMPLES::

    sage: from slabbe import random_jeandel_rao_tiling_rectangle
    sage: tiling = random_jeandel_rao_tiling_rectangle(4, 4)
    sage: tiling
    A wang tiling of a 4 x 4 rectangle
    sage: tiling.table()   # random
    [[1, 10, 4, 5], [1, 3, 3, 7], [0, 9, 10, 4], [0, 9, 3, 3]]

REFERENCES:

.. [Lab2019] S. Labbé. A Markov partition for Jeandel-Rao aperiodic
   Wang tilings. March 2019. https://arxiv.org/abs/1903.06137
"""
#*****************************************************************************
#       Copyright (C) 2017-2019 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import math
from random import random

def plane_to_torus(m,n):
    r"""
    EXAMPLES::

        sage: from slabbe.jeandel_rao import plane_to_torus
        sage: plane_to_torus(0, 0)                       # abs tol 1e-10
        (0.000000000000000, 0.0)
        sage: plane_to_torus(0.324, .324)                # abs tol 1e-10
        (0.324000000000000, 0.324)
        sage: plane_to_torus(12.324, 12.324)             # abs tol 1e-10
        (0.615796067500630, 3.08793202250021)
        sage: plane_to_torus(100, 100)                   # abs tol 1e-10
        (1.3343685400050447, 3.021286236252207)

    """
    phi = .5 + .5 * math.sqrt(5)
    m = float(m)
    n = float(n)
    h = n // float(phi + 3)
    return (m - h) % float(phi), n % float(phi+3)

def torus_to_code(x,y):
    r"""
    Return in which atom of the partition associated to Jeandel-Rao tilings
    the point (x,y) falls in according to [Lab2019]_.

    EXAMPLES::

        sage: from slabbe.jeandel_rao import torus_to_code
        sage: torus_to_code(0,0)
        0
        sage: torus_to_code(0.23,3.5)
        5
        sage: torus_to_code(1.23,2.243)
        3

    ::

        sage: from slabbe.jeandel_rao import plane_to_torus, random_torus_point
        sage: torus_to_code(*plane_to_torus(14.4141, 89.14))
        9
        sage: torus_to_code(*random_torus_point())                  # random
        3

    """
    phi = .5 + .5 * math.sqrt(5)
    assert x >= 0, "x(={}) must be nonnegative".format(x)
    assert y >= 0, "y(={}) must be nonnegative".format(y)
    assert x < phi, "x(={}) must be less then phi".format(x)
    assert y < phi+3, "y(={}) must be less then phi+3".format(y)
    # test
    if y < 1:
        if x < (1./phi):
            if y <= phi*x:
                return 0
            else:
                return 1
        elif x < 1:
            if y <= (phi**2)*x -phi:
                return 0
            else:
                return 1
        else:
            if y <= phi*x - phi:
                return 0
            else:
                return 1
    if y <= phi*x + 1:
        if x <= 1./phi:
            return 9
        else:
            if y <= (phi**2)*x + 1 - phi:
                if x >= (1./phi) and x < 1:
                    return 9
                elif x >= 1:
                    if y <= phi*x + 1 - phi:
                        return 9
                    else:
                        return 3
            else:
                return 8
    if y > (phi*x + 1) and y <= phi*x + 2:
        if x <= 1:
            if y <= (phi**2)*x + 1:
                return 10
            else:
                return 7
        else:
            return 7
    if y > phi*x +2:
        if x <= 1:
            if y <= (phi**2)*x + 2:
                return 4
            if y <= (phi*x + phi +1) and y > (phi**2)*x + 2:
                return 2
            else:
                if y > (phi**2)*x + phi + 2:
                    return 6
                if y > (phi*x + 3) and x > (1./(phi**2)):
                    return 6
                else:
                    return 5
        else:
            return 6

def random_torus_point():
    r"""
    Return a random point in the rectangle `[0,\phi[\times[0,\phi+3[`.

    EXAMPLES::

        sage: from slabbe.jeandel_rao import random_torus_point
        sage: random_torus_point()                  # random
        (0.947478386174632, 2.62013791669977)
        sage: random_torus_point()                  # random
        (0.568010404619112, 0.933319012345482)
        sage: random_torus_point()                  # random
        (1.06782191679796, 4.58930423801758)

    ::

        sage: from slabbe.jeandel_rao import torus_to_code
        sage: torus_to_code(*random_torus_point())                  # random
        3
        sage: torus_to_code(*random_torus_point())                  # random
        7

    """
    phi = .5 + .5 * math.sqrt(5)
    return random() * phi, random() * (phi+3)

def random_jeandel_rao_tiling_rectangle(width, height, start=None):
    r"""
    Returns a jeandel rao tiling of a rectangle associated to a given
    (random) starting position on the torus.

    INPUT:

    - ``width`` -- integer
    - ``height`` -- integer
    - ``start`` -- pair of real numbers (default:``None``), if ``None``
      a random start point is chosen

    EXAMPLES::

        sage: from slabbe.jeandel_rao import random_jeandel_rao_tiling_rectangle
        sage: tiling = random_jeandel_rao_tiling_rectangle(4,4)
        sage: tiling
        A wang tiling of a 4 x 4 rectangle
        sage: tiling.table()   # random
        [[1, 10, 4, 5], [1, 3, 3, 7], [0, 9, 10, 4], [0, 9, 3, 3]]

    """
    if start is None:
        x0,y0 = random_torus_point()
    else: 
        x0,y0 = start
        x0,y0 = plane_to_torus(x0, y0)
    tiling = [[torus_to_code(*plane_to_torus(x0+a,y0+b)) 
                  for b in range(height)]
                  for a in range(width)]
    tiles = [(2,4,2,1), (2,2,2,0), (1,1,3,1), (1,2,3,2), (3,1,3,3), (0,1,3,1), 
             (0,0,0,1), (3,1,0,2), (0,2,1,2), (1,2,1,4), (3,3,1,2)]
    tiles = [map(str,t) for t in tiles]
    from collections import defaultdict
    color = defaultdict(lambda : 'white')
    color.update({0:'white', 1:'red', 2:'cyan', 3:'green', 4:'lightgray'})
    color.update({str(k):v for k,v in color.items()})
    from slabbe.wang_tiles import WangTiling
    return WangTiling(tiling, tiles=tiles, color=color)




# -*- coding: utf-8 -*-
r"""
Digital geometry primitives

Subsets of ZZ^d with the edge relation +e_i and -e_i.

EXAMPLES::

    sage: from slabbe import DiscreteSubset
    sage: DiscreteSubset(dimension=2)
    Subset of ZZ^2
    sage: DiscreteSubset(dimension=4)
    Subset of ZZ^4

A subset from an iterable::

    sage: L = [(0,0,0,0), (1,0,0,0), (2,0,0,0), (3,0,0,0)]
    sage: s = DiscreteSubset.from_subset(L)
    sage: s
    Subset of ZZ^4

A discrete 2d disk::

    sage: D = DiscreteSubset(dimension=2, predicate=lambda p: p[0]^2 + p[1]^2 < 4)
    sage: sorted(D.list())
    [(-1, -1), (-1, 0), (-1, 1), (0, -1), (0, 0), (0, 1), (1, -1), (1, 0), (1, 1)]
    sage: D
    Subset of ZZ^2

A discrete 3d ball::

    sage: predicate = lambda p: p[0]^2 + p[1]^2 + p[2]^2 <= 4
    sage: D = DiscreteSubset(dimension=3, predicate=predicate)
    sage: D
    Subset of ZZ^3
    sage: (0,0,0) in D
    True
    sage: (10,10,10) in D
    False
    sage: len(D.list())
    33
    sage: D.plot()    # optional long

A discrete 4d hyperplane::

    sage: predicate = lambda p: 0 <= 2*p[0] + 3*p[1] + 4*p[2] + 5*p[3] < 14
    sage: D = DiscreteSubset(dimension=4, predicate=predicate)
    sage: D
    Subset of ZZ^4
    sage: D.an_element()
    (0, 0, 0, 0)

A 2d discrete box::

    sage: from slabbe import DiscreteBox
    sage: b = DiscreteBox([-5,5], [-5,5])
    sage: b
    Box: [-5, 5] x [-5, 5]
    sage: b.plot()       # optional long

A 3d discrete box::

    sage: b = DiscreteBox([-2,2], [-5,5], [-5,5])
    sage: b
    Box: [-2, 2] x [-5, 5] x [-5, 5]
    sage: b.plot()       # optional long

The intersection of two discrete objects of the same dimension::

    sage: circ = DiscreteSubset(dimension=2, predicate=lambda p: p[0]^2+p[1]^2<=100)
    sage: b = DiscreteBox([0,10], [0,10])
    sage: I = circ & b
    sage: I
    Intersection of the following objects:
    Subset of ZZ^2
    [0, 10] x [0, 10]
    sage: I.an_element()
    (0, 0)
    sage: I.plot()      # optional long

A discrete tube (preimage of a discrete box by a matrix)::

    sage: from slabbe import M3to2
    sage: M3to2
    [-0.866025403784439  0.866025403784439  0.000000000000000]
    [-0.500000000000000 -0.500000000000000   1.00000000000000]
    sage: from slabbe import DiscreteTube
    sage: tube = DiscreteTube([-5,5],[-5,5], projmat=M3to2)
    sage: tube
    DiscreteTube: Preimage of [-5, 5] x [-5, 5] by a 2 by 3 matrix
    sage: it = iter(tube)
    sage: [next(it) for _ in range(4)]  # random
    [(0, 0, 0), (1, 0, 0), (0, 0, 1), (0, 0, -1)]

TODO:

    - Code Complement
    - The method projection_matrix should be outside of the class?
    - DiscreteTube should have a method projection_matrix
    - Their should be an input saying whether the object is connected or
      not and what kind of neighbor connectedness
"""
#*****************************************************************************
#       Copyright (C) 2010-2016 Sébastien Labbé <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License version 2 (GPLv2)
#
#  The full text of the GPLv2 is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function
import itertools
from copy import copy
from sage.functions.other import sqrt
from sage.matrix.constructor import matrix
from sage.rings.real_mpfr import RR
from sage.modules.free_module_element import vector
from sage.structure.sage_object import SageObject
from sage.modules.free_module import FreeModule
from sage.rings.integer_ring import ZZ
from sage.sets.recursively_enumerated_set import RecursivelyEnumeratedSet
from sage.misc.decorators import options
from sage.misc.cachefunc import cached_method
from sage.misc.functional import round
from sage.misc.latex import LatexExpr
from sage.plot.graphics import Graphics
from sage.plot.point import point
from sage.plot.circle import circle
from sage.plot.plot3d.shapes2 import text3d
from sage.plot.line import line
from sage.plot.text import text
from sage.plot.plot3d.platonic import cube
from slabbe.tikz_picture import TikzPicture
from slabbe.matrices import M3to2

################################################
# Convex boundary of a set of 2d points
################################################
def convex_boundary(L):
    r"""
    EXAMPLES::

        sage: from slabbe.discrete_subset import convex_boundary
        sage: convex_boundary([(3,4), (1,2), (3,5)])
        [(3, 5), (1, 2), (3, 4)]

    """
    from sage.geometry.polyhedron.constructor import Polyhedron
    P = Polyhedron(L)
    from sage.geometry.polyhedron.plot import cyclic_sort_vertices_2d
    return [tuple(v) for v in cyclic_sort_vertices_2d(P.vertices())]
    #if P.ambient_dim() != 2:
    #    raise ValueError("polyhedron ambient dim (=%s) must be <=2" % P.ambient_dim())
    #good = P.plot()._objects[-1]
    #assert isinstance(good, sage.plot.polygon.Polygon), "good not polygon"
    #return zip(good.xdata, good.ydata)
################################################
# Discret subsets of ZZ^d
################################################
class DiscreteSubset(SageObject):
    r"""
    A subset of ZZ^d.

    INPUT:

    - ``dimension`` -- integer, dimension of the space
    - ``predicate`` -- function ZZ^d -> {False, True} (default: ``None``)
    - ``edge_predicate`` -- function ZZ^d,ZZ^d -> {False, True} (default:
      ``None``)
    - ``iterator`` -- function (default: ``None``) returning an iterator of
      points, it must be consistent with the predicate
    - ``roots`` -- list (default: ``None``) of some elements in self. If
      ``iterator`` is not provided, it is used to iterate the elements throught
      connectedness.

    EXAMPLES::

        sage: from slabbe import DiscreteSubset
        sage: DiscreteSubset(dimension=3)
        Subset of ZZ^3

    ::

        sage: p = DiscreteSubset(dimension=3, predicate=lambda x:True)
        sage: p
        Subset of ZZ^3

    ::

        sage: fn = lambda p : p[0]+p[1]<p[2]
        sage: p = DiscreteSubset(dimension=3, predicate=fn, roots=[(0,0,1)])
        sage: p
        Subset of ZZ^3

    ::

        sage: F = lambda p: Integers(7)(2*p[0]+5*p[1])
        sage: edge_predicate = lambda p,s: F(s) < F(s)
        sage: D = DiscreteSubset(dimension=3, edge_predicate=edge_predicate)
        sage: D
        Subset of ZZ^3

    From a list::

        sage: L = [(0,0,0), (1,0,0), (2,0,0), (3,0,0)]
        sage: s = DiscreteSubset.from_subset(L)
        sage: s
        Subset of ZZ^3

    Providing a root may be necessary if zero (the origin) is not inside::

        sage: predicate = lambda p : 4 < p[0]^2 + p[1]^2 < 25
        sage: D = DiscreteSubset(dimension=2, predicate=predicate, roots=[(3,0)])

    TESTS:

    No edges go outside of the box::

        sage: from slabbe import DiscreteBox
        sage: B = DiscreteBox([-1,1],[-1,1])
        sage: len(list(B.edges_iterator()))
        12
        sage: sorted(B.edges_iterator())
        [((-1, -1), (-1, 0)), ((-1, -1), (0, -1)), ((-1, 0), (-1, 1)),
        ((-1, 0), (0, 0)), ((-1, 1), (0, 1)), ((0, -1), (0, 0)), ((0, -1),
        (1, -1)), ((0, 0), (0, 1)), ((0, 0), (1, 0)), ((0, 1), (1, 1)),
        ((1, -1), (1, 0)), ((1, 0), (1, 1))]
    """
    def __init__(self, dimension=3, predicate=None, edge_predicate=None,
            iterator=None, roots=None):
        r"""
        Constructor.

        See class:`DiscreteSubset` for complete documentation.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: DiscreteSubset(dimension=3)
            Subset of ZZ^3
        """
        self._space = FreeModule(ZZ, dimension)
        if predicate is None:
            self._predicate = lambda p: len(p) == dimension
        else:
            self._predicate = predicate
        if edge_predicate is None:
            self._edge_predicate = lambda p,s: p in self and s in self
        else:
            self._edge_predicate = edge_predicate
        self._iterator = iterator
        if roots is None:
            self._roots = None
        else:
            self._roots = [self._space(root) for root in roots]
            for p in self._roots: p.set_immutable()

    @cached_method
    def roots(self):
        r"""
        Return the roots, i.e., a list of elements in self.

        It also makes sure the roots are in self and raises an error otherwise.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: s = DiscreteSubset.from_subset([(0,0,0)])
            sage: s.roots()
            [(0, 0, 0)]

        ::

            sage: predicate = lambda p : 4 < p[0]^2 + p[1]^2 < 25
            sage: D = DiscreteSubset(dimension=2, predicate=predicate, roots=[(3,0)])
            sage: D.roots()
            [(3, 0)]

        TESTS::

            sage: predicate = lambda p : 4 < p[0]^2 + p[1]^2 < 25
            sage: D = DiscreteSubset(dimension=2, predicate=predicate, roots=[(2,0)])
            sage: D.roots()
            Traceback (most recent call last):
            ...
            ValueError: root element (=(2, 0)) provided at initialisation is not in self

        An error is raised if the roots are inconsistent::

            sage: s = DiscreteSubset.from_subset([])
            sage: s.roots()
            Traceback (most recent call last):
            ...
            ValueError: default element (=(0, 0, 0)) is not in self, please provide one at initialisation
        """
        if self._roots is None:
            element = self._space(0)
            element.set_immutable()
            if not element in self: 
                raise ValueError("default element (={}) is not in self, please provide "
                        "one at initialisation".format(element))
            self._roots = [element]
        else:
            for p in self._roots:
                if not p in self: 
                    raise ValueError("root element (={}) provided at"
                            " initialisation is not in self".format(p))
        return self._roots

    @classmethod
    def from_subset(self, subset):
        r"""
        Constructor from a finite subset.

        INPUT:

        - ``subset`` -- iterable of integer coordinate points

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: L = [(0,0,0), (1,0,0), (2,0,0), (3,0,0)]
            sage: s = DiscreteSubset.from_subset(L)
            sage: s
            Subset of ZZ^3
            sage: all(p in s for p in s)
            True

        Note that tuple or mutable vectors work fine::

            sage: (0,0,0) in s
            True
            sage: vector((0,0,0)) in s
            True

        TESTS::

            sage: DiscreteSubset.from_subset([])
            Subset of ZZ^3
        """
        try:
            dimension = len(next(iter(subset)))
        except StopIteration:
            dimension = 3
        space = FreeModule(ZZ, dimension)
        space_subset = set()
        for p in subset:
            p = space(p)
            p.set_immutable()
            space_subset.add(p)
        def predicate(p):
            p = space(p)
            p.set_immutable()
            return p in space_subset
        iterator = space_subset.__iter__
        return DiscreteSubset(dimension=dimension, predicate=predicate,
                iterator=iterator)

    def _repr_(self):
        r"""
        String representation.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: DiscreteSubset(dimension=3)
            Subset of ZZ^3

        ::

            sage: DiscreteSubset(dimension=3, predicate=lambda x:True)
            Subset of ZZ^3

        """
        return "Subset of ZZ^{}".format(self.dimension())

    def dimension(self):
        r"""
        Returns the dimension of the ambiant space.

        OUTPUT:

            integer

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: d = DiscreteSubset(dimension=3)
            sage: d.dimension()
            3

        ::

            sage: from slabbe import DiscreteBox
            sage: p = DiscreteBox([0,3], [0,3], [0,3], [0,3])
            sage: p.dimension()
            4
        """
        return self._space.dimension()

    def __contains__(self, p):
        r"""
        Returns True if the point p is in self.

        INPUT:

        - ``p`` - point in the space

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: d = DiscreteSubset(dimension=3)
            sage: (0,0,0) in d
            True
            sage: (0,0,0,0) in d
            False

        ::

            sage: fn = lambda p : p[0]+p[1]<p[2]
            sage: p = DiscreteSubset(dimension=3, predicate=fn)
            sage: vector((1,2,4)) in p
            True
            sage: vector((1,2,2)) in p
            False
        """
        return self._predicate(p)

    def has_edge(self, p, s):
        r"""
        Returns whether it has the edge (p, s) where s-p is a canonical
        vector.

        INPUT:

        - ``p`` - point in the space
        - ``s`` - point in the space

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: F = lambda p: Integers(7)(2*p[0]+5*p[1])
            sage: edge_predicate = lambda p,s: F(p) < F(s)
            sage: D = DiscreteSubset(dimension=3, edge_predicate=edge_predicate)
            sage: D.has_edge(vector((0,0)),vector((1,0)))
            True
            sage: D.has_edge(vector((0,0)),vector((-1,0)))
            True
            sage: D.has_edge(vector((-1,1)),vector((1,0)))
            False
        """
        return self._edge_predicate(p, s)

    def an_element(self):
        r"""
        Returns an immutable element in self.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: p = DiscreteSubset(dimension=3)
            sage: p.an_element()
            (0, 0, 0)
            sage: p.an_element().is_immutable()
            True

        ::

            sage: predicate = lambda p : 4 < p[0]^2 + p[1]^2 < 25
            sage: D = DiscreteSubset(dimension=2, predicate=predicate, roots=[(3,0)])
            sage: D.an_element()
            (3, 0)
        """
        return self.roots()[0]

    def __and__(self, other):
        r"""
        Return the intersection of self and other.

        INPUT:

        - ``other`` - Discrete object of dimension equal to s.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset, DiscretePlane
            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: p & d3
            Intersection of the following objects:
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
            Subset of ZZ^3

        TESTS::

            sage: d3 = DiscreteSubset(dimension=3)
            sage: d3 & 4
            Traceback (most recent call last):
            ...
            TypeError: Impossible to construct an intersection containing the object : 4

        ::

            sage: d3 = DiscreteSubset(dimension=3)
            sage: d5 = DiscreteSubset(dimension=5)
            sage: d3 & d5
            Traceback (most recent call last):
            ...
            ValueError: Intersection not defined for objects not of the same dimension

        Intersection of intersection do not stack up::

            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: I = p & d3
            sage: p & I
            Intersection of the following objects:
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
            Subset of ZZ^3
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
        """
        if isinstance(other, Intersection):
            return other.__and__(self)
        return Intersection((self, other))

    def __le__(self, other):
        r"""
        Return whether self is a subset of other.

        Runs into an infinite loop if self is infinite.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: P = DiscretePlane([4,6,7], 17, mu=0)
            sage: tube = DiscreteTube([-6.4, 6.4], [-5.2, 5.2])
            sage: I = tube & P
            sage: I <= tube
            True
            sage: I <= P
            True
        """
        return all(p in other for p in self)

    def __ge__(self, other):
        r"""
        Return whether self is a superset of other.

        Runs into an infinite loop if other is infinite.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: P = DiscretePlane([4,6,7], 17, mu=0)
            sage: tube = DiscreteTube([-6.4, 6.4], [-5.2, 5.2])
            sage: I = tube & P
            sage: tube >= I
            True
            sage: P >= I
            True
        """
        return all(p in self for p in other)

    @cached_method
    def base_edges(self):
        r"""
        Return a list of positive canonical vectors.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: d = DiscreteSubset(dimension=2)
            sage: d.base_edges()
            [(1, 0), (0, 1)]

        ::

            sage: from slabbe import DiscretePlane
            sage: P = DiscretePlane([3,4,5], 12)
            sage: P.base_edges()
            [(1, 0, 0), (0, 1, 0), (0, 0, 1)]

        """
        d = self.dimension()
        canonical_base = [copy(self._space.zero()) for _ in range(d)]
        for i in range(d):
            canonical_base[i][i] = 1
        return canonical_base
        #return canonical_base + [-v for v in canonical_base]

    def children(self, p):
        r"""

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: list(p.children(vector((0,0,0))))
            [(1, 0, 0), (0, 1, 0), (0, 0, 1)]
        """
        for v in self.base_edges():
            u = p+v
            u.set_immutable()
            if u in self: yield u
            u = p-v
            u.set_immutable()
            if u in self: yield u

    def d_neighbors(self, p, d=2):
        r"""
        Retourne le voisinage du point p, i.e. les points parmi les 3^d
        possible qui appartiennent a l'objet discret.

        INPUT:

        - ``p`` - un point discret
        - ``d`` - integer (optional, default:2), 

        OUTPUT:

        liste de points

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,3,7], 10)
            sage: p.d_neighbors((0,0,0))
            [(-1, -1, 1), (-1, 0, 1), (-1, 1, 0), (-1, 1, 1), (0, -1, 1),
            (0, 0, 0), (0, 0, 1), (0, 1, 0), (1, -1, 1), (1, 0, 0), (1, 0,
            1), (1, 1, 0)]
        """
        if d != 2:
            raise NotImplementedError("implemented only for d=2")
        p = self._space(p)
        v = [-1, 0, 1]
        L = []
        for a in itertools.product(*[v]*self.dimension()):
            b = p + self._space(a)
            if b in self:
                L.append(b)
        return L

    def connected_component_iterator(self, roots=None):
        r"""
        Return an iterator over the connected component of the root.

        INPUT:

        - ``roots`` - list of some elements immutable in self

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: p = DiscreteSubset(dimension=3, roots=[(0,0,0)])
            sage: it = p.connected_component_iterator()
            sage: [next(it) for _ in range(5)]     # random
            [(0, 0, 0), (1, 0, 0), (0, 0, 1), (0, 0, -1), (0, -1, 0)]

        ::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: root = vector((0,0,0))
            sage: root.set_immutable()
            sage: it = p.connected_component_iterator(roots=[root])
            sage: [next(it) for _ in range(5)]     # random
            [(0, 0, 0), (1, 0, 0), (0, 1, 0), (0, 0, 1), (-1, 1, 0)]
        """
        roots = roots if roots else self.roots()
        if not all(root in self for root in roots):
            raise ValueError("roots (=%s) must all be in self(=%s)" % (roots, self))
        #for root in roots:
        #    root.set_immutable()
        C = RecursivelyEnumeratedSet(seeds=roots, successors=self.children, structure='symmetric')
        return C.breadth_first_search_iterator()

    def __iter__(self):
        r"""
        Return an iterator over self.

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: it = iter(p)
            sage: [next(it) for _ in range(5)]  # random
            [(0, 0, 0), (1, 0, 0), (0, 1, 0), (0, 0, 1), (-1, 1, 0)]

        ::

            sage: from slabbe import DiscreteSubset
            sage: L = [(0,0,0), (1,0,0), (2,0,0), (3,0,0)]
            sage: s = DiscreteSubset.from_subset(L)
            sage: sorted(s)
            [(0, 0, 0), (1, 0, 0), (2, 0, 0), (3, 0, 0)]
            sage: all(p in s for p in s)
            True
        """
        if self._iterator is None:
            return self.connected_component_iterator()
        else:
            return self._iterator()

    def list(self):
        r"""
        Return the list of elements in self.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: P = DiscretePlane([3,4,5], 12, mu=20)
            sage: tube = DiscreteTube([0,2],[0,2])
            sage: I = P & tube
            sage: sorted(I.list())
            [(-3, -1, -1), (-3, -1, 0), (-2, -2, -1), (-2, -2, 0), (-2, -1,
            -1), (-2, -1, 0), (-2, 0, -1), (-1, -1, -1)]
        """
        return list(self)

    def level_iterator(self):
        r"""
        This returns an iterator of the levels according to the given roots.

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: p = DiscreteSubset(dimension=3, roots=[(0,0,0)])
            sage: it = p.level_iterator()
            sage: sorted(next(it))
            [(0, 0, 0)]
            sage: sorted(next(it))
            [(-1, 0, 0), (0, -1, 0), (0, 0, -1), (0, 0, 1), (0, 1, 0), (1, 0, 0)]
            sage: sorted(next(it))
            [(-2, 0, 0),
             (-1, -1, 0),
             (-1, 0, -1),
             (-1, 0, 1),
             (-1, 1, 0),
             (0, -2, 0),
             (0, -1, -1),
             (0, -1, 1),
             (0, 0, -2),
             (0, 0, 2),
             (0, 1, -1),
             (0, 1, 1),
             (0, 2, 0),
             (1, -1, 0),
             (1, 0, -1),
             (1, 0, 1),
             (1, 1, 0),
             (2, 0, 0)]

        ::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: it = p.level_iterator()
            sage: sorted(next(it))
            [(0, 0, 0)]
            sage: sorted(next(it))
            [(0, 0, 1), (0, 1, 0), (1, 0, 0)]
            sage: sorted(next(it))
            [(-1, 0, 1),
             (-1, 1, 0),
             (0, -1, 1),
             (0, 1, 1),
             (0, 2, 0),
             (1, 0, 1),
             (1, 1, 0),
             (2, 0, 0)]
        """
        roots = self.roots()
        t = RecursivelyEnumeratedSet(seeds=roots, successors=self.children, structure='symmetric')
        return t.graded_component_iterator()

    def edges_iterator(self):
        r"""
        Returns an iterator over the pair of points in self that are
        adjacents, i.e. their difference is a canonical vector.

        It considers only points that are connected to the given roots.

        INPUT:

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: it = p.edges_iterator()
            sage: next(it)
            ((0, 0, 0), (1, 0, 0))
            sage: next(it)
            ((0, 0, 0), (0, 1, 0))
            sage: next(it)
            ((0, 0, 0), (0, 0, 1))
            sage: next(it)         # random
            ((-1, 1, 0), (0, 1, 0))
            sage: next(it)         # random
            ((-2, 1, 0), (-1, 1, 0))
        """
        base_edges = self.base_edges()
        for p in self:
            # astuce pour eviter les doublons pour cette connexite
            if sum(p)%2 != 0: continue
            for e in base_edges:
                s = p+e
                if self.has_edge(p, s):
                    yield (p, s)
                s = p-e
                if self.has_edge(s, p):
                    yield (s, p)

    def projection_matrix(self, m='isometric', oblique=None):
        r"""
        Return a projection matrix.

        INPUT:

        - ``m`` -- projection matrix (default: ``'isometric'``), it can be one of
          the following:
        
          - ``'isometric'`` - the isometric projection is used by default
          - matrix - a 2 x 3 matrix
          - ``'belle'`` - shortcut for ``matrix(2, [1/3.0, 1, 0, 2/3.0, 0, 1])``
          - vector - defines the projection on the plane orthogonal to the
            vector.

        - ``oblique`` -- vector (default: ``None``), vector perpendicular
          to the range space

        EXAMPLES::

            sage: from slabbe import DiscreteSubset
            sage: d = DiscreteSubset(dimension=3)
            sage: d.projection_matrix(vector((2,3,4))) # tolerance 0.00001
            [  1.00000000000000  0.000000000000000 -0.500000000000000]
            [ 0.000000000000000   1.00000000000000 -0.750000000000000]
            sage: d.projection_matrix((2,3,4)) # tolerance 0.00001
            [  1.00000000000000  0.000000000000000 -0.500000000000000]
            [ 0.000000000000000   1.00000000000000 -0.750000000000000]
            sage: d.projection_matrix()         # tolerance 0.00001
            [-0.866025403784  0.866025403784             0.0]
            [           -0.5            -0.5             1.0]
            sage: d.projection_matrix(_) # tolerance 0.00001
            [-0.866025403784439  0.866025403784439  0.000000000000000]   
            [-0.500000000000000 -0.500000000000000   1.00000000000000]   
            sage: d.projection_matrix('belle')    # tolerance 0.00001
            [0.333333333333            1.0            0.0]
            [0.666666666667            0.0            1.0]
        """
        if m == 'isometric':
            return matrix(2, [-1.7320508075688772*0.5, 1.7320508075688772*0.5, 0, 
                                           -0.5, -0.5, 1])
        elif m == 'belle':
            return matrix(2, [1/3.0, 1, 0, 2/3.0, 0, 1])
        elif m == 'plusbelle':
            return matrix(2, [1, 0, -0.4, 0, 1, -0.4])
        else:
            try:
                v = matrix(RR, m)
            except:
                raise
                #TypeError, "unknown input for m(=%s)" % m
            dim = v.ncols(), v.nrows()
            if dim == (3,2):
                return v
            elif dim == (3,1):
                return v.right_kernel().basis_matrix()
            else:
                raise ValueError("uncorrect dimension (=%s) " % (dim,))

    def plot_points(self, color='blue', m=None):
        r"""
        Returns a 2d or 3d graphics object of the points of self.

        INPUT:

        - ``color`` -- string (default: ``'blue'``), the color of the points
        - ``m`` -- projection matrix (default: ``None``), it can be one of
          the following:
        
          - ``None`` - no projection is done
          - ``'isometric'`` - the isometric projection
          - matrix - a 2 x n projection matrix
          - ``'belle'`` - shortcut for ``matrix(2, [1/3.0, 1, 0, 2/3.0, 0, 1])``
          - vector - defines the projection on the plane orthogonal to the
            vector.

        EXAMPLES:

        A 2d plot of a 2d object::

            sage: from slabbe import DiscreteSubset, DiscreteBox
            sage: D = DiscreteSubset(dimension=2)
            sage: box = DiscreteBox([-5,5],[-5,5])
            sage: I = D & box
            sage: I.plot_points(color='green')      # optional long

        A 3d plot of a 3d object::

            sage: D = DiscreteSubset(dimension=3)
            sage: box = DiscreteBox([-5,5],[-5,5],[-5,5])
            sage: I = D & box
            sage: I.plot_points(color='green')      # optional long

        A 2d plot of a 3d object::

            sage: D = DiscreteSubset(dimension=3)
            sage: box = DiscreteBox([-5,5],[-5,5],[-5,5])
            sage: I = D & box
            sage: I.plot_points(color='green', m='isometric')      # optional long
        """
        if m is None:
            L = self.list()
        else:
            m = self.projection_matrix(m=m)
            L = [m*p for p in self]
        if not all(len(p) in [2,3] for p in L[:5]):
            raise ValueError("dimension of points after projection must be 2 or 3")
        G = point(L, color=color)
        #G.set_aspect_ratio(1) # for 2d only
        #G.axes(False)         # for 2d only
        return G

    def plot_points_at_distance(self, k, color='blue', projmat=None):
        r"""
        Plot points at distance k from the roots.

        INPUT:

        - ``k`` - integer

        EXAMPLES::

            sage: alpha = solve(x+x**2+x**3==1, x)[2].right()
            sage: vv = vector((alpha, alpha+alpha**2, 1))
            sage: omega = (1+alpha)**2 / 2
            sage: from slabbe import DiscretePlane
            sage: Pr = DiscretePlane(vv, omega, mu=pi, prec=200)
            sage: Pr.plot_points_at_distance(200)               # optional long
            sage: Pr.plot_points_at_distance(200, projmat='isometric') # optional long
        """
        it = self.level_iterator()
        G = Graphics()
        for i in range(k):
            B = next(it)
        if projmat is None:
            L = [p for p in B]
        else:
            m = self.projection_matrix(m=projmat)
            L = [m * p for p in B]
            roots = [m * root for root in self.roots()]
        G += point(L, color='black')
        for root in roots:
            G += circle(root, 0.5, color='red', thickness=4)
        return G

    def plot_edges(self, color='blue', m=None):
        r"""
        Returns the mesh of the plane. The mesh is the union of segments
        joining two adjacents points.

        INPUT:

        - ``color`` -- string (default: ``'blue'``), the color of the edges
        - ``m`` -- projection matrix (default: ``None``), it can be one of
          the following:
        
          - ``None`` - no projection is done
          - ``'isometric'`` - the isometric projection
          - matrix - a 2 x 3 matrix
          - ``'belle'`` - shortcut for ``matrix(2, [1/3.0, 1, 0, 2/3.0, 0, 1])``
          - vector - defines the projection on the plane orthogonal to the
            vector.

        EXAMPLES:

        A 2d plot of a 2d object::

            sage: from slabbe import DiscreteSubset, DiscreteBox
            sage: D = DiscreteSubset(dimension=2)
            sage: box = DiscreteBox([-5,5],[-5,5])
            sage: I = D & box
            sage: I.plot_edges(color='green') # optional long

        A 3d plot of a 3d object::

            sage: D = DiscreteSubset(dimension=3)
            sage: box = DiscreteBox([-3,3],[-3,3],[-3,3])
            sage: I = D & box
            sage: I.plot_edges(color='green') # optional long

        A 2d plot of a 3d object::

            sage: D = DiscreteSubset(dimension=3)
            sage: box = DiscreteBox([-3,3],[-3,3],[-3,3])
            sage: I = D & box
            sage: I.plot_edges(color='green', m='isometric') # optional long
        """
        if m is None:
            L = self.edges_iterator()
        else:
            m = self.projection_matrix(m=m)
            L = [(m*p,m*q) for (p,q) in self.edges_iterator()]
        G = Graphics()
        for edge in L:
                G += line(edge, color=color)
        # G.axes(False) # for 2d only
        return G
    def plot_cubes(self, **kwds):
        r"""
        Returns the discrete object as cubes in 3d.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: P = DiscretePlane([3,4,5], 12, mu=20)
            sage: tube = DiscreteTube([-5,5],[-5,5])
            sage: I = P & tube
            sage: I.plot_cubes(color='red', frame_thickness=1 # optional long)

        TESTS::

            sage: from slabbe import DiscreteBox
            sage: box = DiscreteBox([-5,5],[-5,5])
            sage: box.plot_cubes() # optional long
            Traceback (most recent call last):
            ...
            ValueError: this method is currently implemented only for objects living in 3 dimensions

        """
        if self.dimension() != 3:
            raise ValueError("this method is currently implemented only for "
                             "objects living in 3 dimensions")
        p = sum(cube(p, **kwds) for p in self)
        return p

    def plot(self, frame=False, edgecolor="blue", pointcolor="blue"):
        r"""
        Return a plot (2d or 3d) of the points and edges of self.

        INPUT:

        - ``frame`` - (default: False) if True, draw a bounding frame with
          labels
        - ``edgecolor`` -- string (default: ``'blue'``), the color of the edges
        - ``pointcolor`` -- string (default: ``'blue'``), the color of the points

        EXAMPLES:

        2d example::

            sage: from slabbe import DiscreteBox
            sage: box = DiscreteBox([-5,5],[-5,5])
            sage: box.plot() # optional long

        3d example::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: P = DiscretePlane([1,3,7], 11)
            sage: tube = DiscreteTube([-5,5],[-5,5])
            sage: I = P & tube
            sage: I.plot() # optional long
        """
        G = self.plot_edges(color=edgecolor) 
        G += self.plot_points(color=pointcolor)
        if self.dimension() == 2:
            G += text(repr(self), (0.5,1.1), axis_coords=True)
            G.axes(False)
            G.set_aspect_ratio(1)
        elif self.dimension() == 3:
            G += text3d(repr(self), (-2,-2,-2))
        else:
            raise NotImplementedError("implemented only for dimension(=%s) 2 or 3" % self.dimension())
        return G

    def tikz_projection_scale(self, projmat='isometric', scale=1, extra=""):
        r"""
        INPUT:

        - ``projmat`` -- (default: ``'isometric'``) It can be one of the following:

          - ``'isometric'`` - the isometric projection is used by default
          - matrix - a 2 x 3 matrix
          - ``'belle'`` - shortcut for ``matrix(2, [1/3.0, 1, 0, 2/3.0, 0, 1])``
          - vector - defines the projection on the plane orthogonal to the
            vector.

        - ``scale`` -- real number (default: 1), scaling constant for the
          whole figure

        - ``extra`` -- string (default: ``''``)

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([1,3,7], 11)
            sage: p.tikz_projection_scale()
            [x={(-0.866025cm,-0.500000cm)}, y={(0.866025cm,-0.500000cm)},
            z={(0.000000cm,1.000000cm)}, scale=1]
            sage: p.tikz_projection_scale(extra="xshift=4cm")
            [x={(-0.866025cm,-0.500000cm)}, y={(0.866025cm,-0.500000cm)},
            z={(0.000000cm,1.000000cm)}, scale=1,xshift=4cm]

        """
        if self.dimension() != 3:
            raise NotImplementedError("implemented only for dimension (=%s) 3" % self.dimension())

        projmat = self.projection_matrix(projmat)

        e1 = projmat*vector([1,0,0])
        e2 = projmat*vector([0,1,0])
        e3 = projmat*vector([0,0,1])

        t = (e1[0], e1[1], e2[0], e2[1], e3[0], e3[1], scale)
        s = 'x={(%fcm,%fcm)}, y={(%fcm,%fcm)}, \nz={(%fcm,%fcm)}, scale=%s' % t
        if extra:
            s += "," + extra
        return LatexExpr("[%s]\n" % s)

    def tikz_axes(self, xshift=0, yshift=0, label="e", projmat='isometric'):
        r"""
        Return the tikz code for drawing axes.

        INPUT:

        - ``xshift`` - integer (default: ``0``), x shift
        - ``yshift`` - integer (default: ``0``), y shift
        - ``label`` - string (default: ``"e"``), label for base vectors
        - ``projmat`` - matrix (default: ``'isometric'``), projection matrix

        OUTPUT:

        string

        EXAMPLES:

        2d example::

            sage: from slabbe import DiscreteSubset
            sage: d = DiscreteSubset(dimension=2)
            sage: d.tikz_axes()
            %the axes
            \begin{scope}[xshift=0cm,yshift=0cm]
            \draw[->,>=latex, very thick, blue] (0,0) -- (1, 0);
            \draw[->,>=latex, very thick, blue] (0,0) -- (0, 1);
            \node at (1.40000000000000,0) {$e_1$};
            \node at (0,1.40000000000000) {$e_2$};
            \end{scope}

        3d example::

            sage: d = DiscreteSubset(dimension=3)
            sage: d.tikz_axes(projmat='isometric')
            %the axes
            \begin{scope}
            [x={(-0.866025cm,-0.500000cm)}, y={(0.866025cm,-0.500000cm)}, 
            z={(0.000000cm,1.000000cm)}, scale=1,xshift=0,yshift=0]
            \draw[fill=white] (2,0,0) rectangle (-1.8,.1,1);
            \draw[->,>=latex, very thick, blue] (0,0,0) -- (1, 0, 0);
            \draw[->,>=latex, very thick, blue] (0,0,0) -- (0, 1, 0);
            \draw[->,>=latex, very thick, blue] (0,0,0) -- (0, 0, 1);
            \node at (1.40000000000000,0,0) {$e_1$};
            \node at (0,1.40000000000000,0) {$e_2$};
            \node at (0,0,1.40000000000000) {$e_3$};
            \end{scope}

        """
        if self.dimension() == 2:
            s = "%the axes\n"
            s += "\\begin{scope}[xshift=%scm,yshift=%scm]\n" % (xshift, yshift)
            s += "\\draw[->,>=latex, very thick, blue] (0,0) -- (1, 0);\n"
            s += "\\draw[->,>=latex, very thick, blue] (0,0) -- (0, 1);\n"
            s += "\\node at (1.40000000000000,0) {$%s_1$};\n" % label
            s += "\\node at (0,1.40000000000000) {$%s_2$};\n" % label
            s += "\\end{scope}\n"
            return LatexExpr(s)
        elif self.dimension() == 3:
            s = "%the axes\n"
            s += "\\begin{scope}\n"
            extra = "xshift=%s,yshift=%s" % (xshift, yshift)
            s += self.tikz_projection_scale(projmat, scale=1, extra=extra)
            s += "\\draw[fill=white] (2,0,0) rectangle (-1.8,.1,1);\n"
            s += "\\draw[->,>=latex, very thick, blue] (0,0,0) -- (1, 0, 0);\n"
            s += "\\draw[->,>=latex, very thick, blue] (0,0,0) -- (0, 1, 0);\n"
            s += "\\draw[->,>=latex, very thick, blue] (0,0,0) -- (0, 0, 1);\n"
            s += "\\node at (1.40000000000000,0,0) {$%s_1$};\n" % label
            s += "\\node at (0,1.40000000000000,0) {$%s_2$};\n" % label
            s += "\\node at (0,0,1.40000000000000) {$%s_3$};\n" % label
            s += "\\end{scope}\n"
            return LatexExpr(s)
        else:
            raise ValueError("this method is currently implemented only for "
                             "objects living in 2 or 3 dimensions")

    def tikz_edges(self, style='very thick', color='blue', projmat=None):
        r"""
        Returns the mesh of the object. The mesh is the union of segments
        joining two adjacents points.

        INPUT:

        - ``style`` - string (default: ``'dashed, very thick'``)
        - ``color`` - string or callable (default: ``'blue'``), the color
          of all edges or a function : (u,v) -> color of the edge (u,v)
        - ``projmat`` - matrix (default: ``None``), projection matrix, if
          None, no projection is done.

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([2,3,5], 4)
            sage: p.tikz_edges()
            \draw[very thick, blue] (0, 0, 0) -- (1, 0, 0);
            \draw[very thick, blue] (0, 0, 0) -- (0, 1, 0);
            \draw[very thick, blue] (-1, 1, 0) -- (0, 1, 0);

        ::

            sage: p.tikz_edges(color='orange')
            \draw[very thick, orange] (0, 0, 0) -- (1, 0, 0);
            \draw[very thick, orange] (0, 0, 0) -- (0, 1, 0);
            \draw[very thick, orange] (-1, 1, 0) -- (0, 1, 0);

        ::

            sage: c = lambda u,v: 'red' if u == 0 else 'blue'
            sage: p.tikz_edges(color=c)
            \draw[very thick, red] (0, 0, 0) -- (1, 0, 0);
            \draw[very thick, red] (0, 0, 0) -- (0, 1, 0);
            \draw[very thick, blue] (-1, 1, 0) -- (0, 1, 0);

        ::

            sage: from slabbe.discrete_subset import M3to2
            sage: p.tikz_edges(projmat=M3to2)
            \draw[very thick, blue] (0.00000, 0.00000) -- (-0.86603, -0.50000);
            \draw[very thick, blue] (0.00000, 0.00000) -- (0.86603, -0.50000);
            \draw[very thick, blue] (1.73205, 0.00000) -- (0.86603, -0.50000);

        """
        s = ''
        if isinstance(color, str):
            color_f = lambda u,v: color
        else:
            color_f = color
        for (p, q) in self.edges_iterator():
            c = color_f(p,q)
            if projmat:
                p = projmat * p
                q = projmat * q
                p = "(" + ", ".join("%.5f" % a for a in p) + ")"
                q = "(" + ", ".join("%.5f" % a for a in q) + ")"
            s += "\\draw[%s, %s] %s -- %s;\n" % (style, c,  p, q)
        return LatexExpr(s)

    def tikz_points(self, size='0.8mm', label=None, label_pos='right',
            fill='black', options="", filter=None,
            projmat=None):
        r"""
        INPUT:

        - ``size`` - string (default: ``'0.8mm'``), size of the
          points
        - ``label`` - function (default: ``None``), print some label next
          to the point
        - ``label_pos`` - function (default: ``'right'``), tikz label
          position
        - ``fill`` - string (default: ``'black'``), fill color
        - ``options`` - string (default: ``''``), author tikz node circle options
        - ``filter`` - boolean function, if filter(p) is False, the point p
          is not drawn
        - ``projmat`` - matrix (default: ``None``), projection matrix, if
          None, no projection is done.

        EXAMPLES::

            sage: from slabbe import DiscreteBox
            sage: p = DiscreteBox([0,3], [0,3], [0,3])
            sage: s = p.tikz_points()
            sage: lines = s.splitlines()
            sage: lines[0]
            '\\node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};'

        ::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: p = DiscretePlane([1,3,7], 11)
            sage: d = DiscreteTube([-1,1],[-1,1])
            sage: I = p & d
            sage: I.tikz_points()
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (...) {};


        Using a filter on the points::

            sage: I.tikz_points(filter=lambda x:sum(x)==1)
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (1, 0, 0) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (0, 1, 0) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (0, 0, 1) {};

        Of a finite subset::

            sage: from slabbe import DiscreteSubset
            sage: V = [(0,0,0), (1,1,0), (1,-1,1), (-2,1,0), (2,0,1), (-1,2,0),
            ....:      (-1,0,1), (0,1,1)]
            sage: s = DiscreteSubset.from_subset(V)
            sage: s.tikz_points()
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (-2, 1, 0) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (1, 1, 0) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (0, 1, 1) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (-1, 2, 0) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (-1, 0, 1) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (0, 0, 0) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (1, -1, 1) {};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (2, 0, 1) {};
        """
        it = iter(self)
        if filter:
            it = (a for a in it if filter(a))
        s = ''
        if projmat:
            for pt in it:
                pt2d = projmat * pt
                pt2d = "(%.5f, %.5f)" % tuple(pt2d)
                s += '\\node[circle,fill=%s,draw=black,' % fill
                s += 'minimum size=%s,inner sep=0pt,%s] at %s {};\n' % (size, options, pt2d)
                if label:
                    s += '\\node[%s] at %s {$%s$};\n' % (label_pos, pt2d, label(pt))
        else:
            for pt in it:
                s += '\\node[circle,fill=%s,draw=black,' % fill
                s += 'minimum size=%s,inner sep=0pt,%s] at %s {};\n' % (size, options, pt)
                if label:
                    s += '\\node[%s] at %s {$%s$};\n' % (label_pos, pt, label(pt))
        return LatexExpr(s)

    def tikz_noprojection(self, projmat=None, scale=1, clip=[], edges=True,
            points=True, axes=False, point_kwds={}, edge_kwds={}, axes_kwds={}, extra_code=''):
        r"""
        Return the tikz code of self.
        
        In this version, the points are not projected. If the points are in
        3d, the tikz 3d picture is used.

        INPUT:

        - ``projmat`` -- (default: None) 2*3 projection matrix
          for drawing unit faces, the isometric projection is used by
          default
        - ``scale`` -- real number (default: 1), scaling constant for the
          whole figure
        - ``clip`` - list (default: ``[]``), list of points describing a
          cliping path once projected. Works only if ``self.dimension()``
          is 2.
        - ``edges`` - bool (default: ``True``), whether to draw edges
        - ``points`` - bool (default: ``True``), whether to draw points
        - ``axes`` - bool (default: ``False``), whether to draw axes
        - ``point_kwds`` - dict (default: ``{}``)
        - ``edge_kwds`` - dict (default: ``{}``)
        - ``axes_kwds`` - dict (default: ``{}``)
        - ``extra_code`` -- string (default: ``''``), extra tikz code to add

        EXAMPLES:

        Object in 2d::

            sage: from slabbe import DiscreteLine, DiscreteBox
            sage: L = DiscreteLine([2,5], 2+5, mu=0)
            sage: b = DiscreteBox([-5,5],[-5,5])
            sage: I = L & b
            sage: point_kwds = {'label':lambda p:2*p[0]+5*p[1],'label_pos':'above right'}
            sage: tikz = I.tikz_noprojection(scale=0.5,point_kwds=point_kwds)
            sage: tikz
            \documentclass[tikz]{standalone}
            \usepackage{amsmath}
            \begin{document}
            \begin{tikzpicture}
            [scale=0.500000000000000]
            \draw[very thick, blue] (0, 0) -- (1, 0);
            \draw[very thick, blue] (0, 0) -- (0, 1);
            \draw[very thick, blue] (2, 0) -- (3, 0);
            ...
            ... 40 lines not printed (2659 characters in total) ...
            ...
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (-5, 2) {};
            \node[above right] at (-5, 2) {$0$};
            \node[circle,fill=black,draw=black,minimum size=0.8mm,inner sep=0pt,] at (-5, 3) {};
            \node[above right] at (-5, 3) {$5$};
            \end{tikzpicture}
            \end{document}

        Object in 3d::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: p = DiscretePlane([1,3,7], 11)
            sage: d = DiscreteTube([-5,5],[-5,5])
            sage: I = p & d
            sage: s = I.tikz_noprojection()
            sage: s
            \documentclass[tikz]{standalone}
            \usepackage{amsmath}
            \begin{document}
            \begin{tikzpicture}
             [x={(-0.866025cm,-0.500000cm)}, y={(0.866025cm,-0.500000cm)},
            z={(0.000000cm,1.000000cm)}, scale=1]
            \draw[very thick, blue] (0, 0, 0) -- (1, 0, 0);
            \draw[very thick, blue] (0, 0, 0) -- (0, 1, 0);
            ...
            ... 311 lines not printed (20339 characters in total) ...
            ...
            \end{tikzpicture}
            \end{document}
        """
        s = '\\begin{tikzpicture}\n'
        if self.dimension() == 2:
            s += "[scale=%s]\n" % scale
        elif self.dimension() == 3:
            if projmat is None:
                projmat = 'isometric'
            s += self.tikz_projection_scale(projmat, scale)
        else:
            raise NotImplementedError("implemented only for dimension(=%s) 2 or 3" % self.dimension())
        if axes:
            s += self.tikz_axes(**axes_kwds)
        if clip and self.dimension() == 2:
            ss = " -- ".join(str(pt) for pt in clip)
            #s += "\\draw %s -- cycle;" % ss
            s += "\\clip %s -- cycle;" % ss
        if edges:
            s += self.tikz_edges(**edge_kwds)
        if points:
            s += self.tikz_points(**point_kwds)
        s += extra_code
        s += '\\end{tikzpicture}\n'
        return TikzPicture(s)

    def tikz(self, projmat=M3to2, scale=1, clip=[], contour=[],
            edges=True, points=True, axes=False, point_kwds={},
            edge_kwds={}, axes_kwds={}, extra_code=''):
        r"""
        INPUT:

        - ``projmat`` -- (default: M3to2) 2 x dim projection
          matrix where dim is the dimensoin of self, the isometric
          projection is used by default
        - ``scale`` -- real number (default: 1), scaling constant for the
          whole figure
        - ``clip`` - list (default: ``[]``), list of points whose convex
          hull describes a cliping path
        - ``contour`` - list (default: ``[]``), list of points describing a
          contour path to be drawn
        - ``edges`` - bool (default: ``True``), whether to draw edges
        - ``points`` - bool (default: ``True``), whether to draw points
        - ``axes`` - bool (default: ``False``), whether to draw axes
        - ``point_kwds`` - dict (default: ``{}``)
        - ``edge_kwds`` - dict (default: ``{}``)
        - ``axes_kwds`` - dict (default: ``{}``)
        - ``extra_code`` -- string (default: ``''``), extra tikz code to add

        EXAMPLES::

            sage: from slabbe import DiscretePlane
            sage: p = DiscretePlane([2,3,5], 10)
            sage: print(p.tikz(points=False, edges=False))
            \documentclass[tikz]{standalone}
            \usepackage{amsmath}
            \begin{document}
            \begin{tikzpicture}
            [scale=1]
            \end{tikzpicture}
            \end{document}
        """
        s = '\\begin{tikzpicture}\n'
        s += '[scale=%s]\n' % scale
        #clip
        if clip:
            if len(clip[0]) == 2:
                pass
            elif len(clip[0]) == projmat.ncols() and projmat.nrows() == 2:
                clip = [projmat * c for c in clip]
            else:
                raise ValueError("incorrect space dimension of clip data (=%s)" %len(clip[0]))
            clip_convex = convex_boundary(clip)
            clip_s = " -- ".join(str(pt) for pt in clip_convex)
            s += "\\clip %s -- cycle;\n" % clip_s
        # edges
        if edges:
            s += self.tikz_edges(projmat=projmat,**edge_kwds)
        # points
        if points:
            s += self.tikz_points(projmat=projmat,**point_kwds)
        # contour
        if contour:
            if 2 != len(contour[0]) == projmat.ncols() and projmat.nrows() == 2:
                contour = [projmat * c for c in contour]
            else:
                raise ValueError("incorrect space dimension of contour data (=%s)" %len(contour[0]))
            contour_convex = convex_boundary(contour)
            contour_s = " -- ".join(str(pt) for pt in contour_convex)
            if clip:
                s += "\\filldraw[fill=white,very thick,dotted,opacity=0.5,even odd rule]\n"
                s += "  %s -- cycle\n" % clip_s
                s += "  %s -- cycle;\n" % contour_s
            else:
                s += "\\draw %s -- cycle;\n" % contour_s
        s += extra_code
        # axes
        if axes:
            s += self.tikz_axes(projmat=projmat,**axes_kwds)
        s += '\\end{tikzpicture}\n'
        return TikzPicture(s)

class Intersection(DiscreteSubset):
    r"""
    Intersection

    todo:

    - Rendre l'heritage 3d automatique

    INPUT:

    - ``objets`` - un tuple d'objets discrets

    EXAMPLES:

    Intersection de deux plans::

        sage: from slabbe import DiscretePlane, Intersection
        sage: p = DiscretePlane([1,3,7],11)
        sage: q = DiscretePlane([1,3,5],9)
        sage: Intersection((p,q))
        Intersection of the following objects:
        Set of points x in ZZ^3 satisfying: 0 <= (1, 3, 7) . x + 0 < 11
        Set of points x in ZZ^3 satisfying: 0 <= (1, 3, 5) . x + 0 < 9

    Shortcut::

        sage: p & q
        Intersection of the following objects:
        Set of points x in ZZ^3 satisfying: 0 <= (1, 3, 7) . x + 0 < 11
        Set of points x in ZZ^3 satisfying: 0 <= (1, 3, 5) . x + 0 < 9

    Intersection of a plane and a tube::

        sage: from slabbe import DiscreteTube
        sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
        sage: d = DiscreteTube([-5,5],[-5,5])
        sage: I = p & d
        sage: I
        Intersection of the following objects:
        Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
        DiscreteTube: Preimage of [-5, 5] x [-5, 5] by a 2 by 3 matrix
        sage: len(list(I))
        115

    Intersection of a line and a box::

        sage: from slabbe import DiscreteLine, DiscreteBox
        sage: L = DiscreteLine([2,5], 2+5, mu=0)
        sage: b = DiscreteBox([-5,5],[-5,5])
        sage: I = L & b
        sage: I
        Intersection of the following objects:
        Set of points x in ZZ^2 satisfying: 0 <= (2, 5) . x + 0 < 7
        [-5, 5] x [-5, 5]

    TESTS:

    Intersected objects must be of the same dimension::

        sage: box = DiscreteBox([-5,5],[-5,5])
        sage: p = DiscretePlane([1,pi,7], 1+pi+7)
        sage: p & box
        Traceback (most recent call last):
        ...
        ValueError: Intersection not defined for objects not of the same dimension

    """
    def __init__(self, objets):
        r"""
        Constructeur d'intersection d'objets.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteSubset
            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: I = p & d3
            sage: I
            Intersection of the following objects:
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
            Subset of ZZ^3

        TESTS::

            sage: from slabbe import Intersection
            sage: type(Intersection([DiscreteSubset(dimension=2)]))
            <class 'slabbe.discrete_subset.Intersection'>
            sage: type(Intersection([DiscreteSubset(dimension=3)]))
            <class 'slabbe.discrete_subset.Intersection'>
            sage: type(Intersection([DiscreteSubset(dimension=4)]))
            <class 'slabbe.discrete_subset.Intersection'>
        """
        for o in objets:
            if not isinstance(o, DiscreteSubset):
                raise TypeError("Impossible to construct an intersection "
                        "containing the object : {}".format(o))
        dimension = objets[0].dimension()
        if not all(o.dimension() == dimension for o in objets):
            raise ValueError("Intersection not defined for objects not of the same dimension")
        self._objets = objets
        DiscreteSubset.__init__(self, dimension=dimension)
    @cached_method
    def roots(self):
        r"""
        EXAMPLES::

            sage: from slabbe import DiscreteBox, DiscreteSubset
            sage: d3 = DiscreteSubset(dimension=3, roots=[(0,0,0), (1,1,1)])
            sage: box = DiscreteBox([-5,5],[-5,5],[-5,5])
            sage: I = d3 & box
            sage: sorted(d3.roots())
            [(0, 0, 0), (1, 1, 1)]
            sage: box.roots()
            [(0, 0, 0)]
            sage: sorted(I.roots())
            [(0, 0, 0), (1, 1, 1)]
        """
        s = set()
        for o in self._objets:
            s.update(o.roots())
        return [p for p in s if p in self]

    def _repr_(self):
        r"""
        Retourne la représentation de l'objet en chaîne de caractères.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, Intersection
            sage: p = DiscretePlane([1,3,7], 11)
            sage: q = DiscretePlane([1,3,5], 9)
            sage: Intersection((p,q))
            Intersection of the following objects:
            Set of points x in ZZ^3 satisfying: 0 <= (1, 3, 7) . x + 0 < 11
            Set of points x in ZZ^3 satisfying: 0 <= (1, 3, 5) . x + 0 < 9
        """
        s = "Intersection of the following objects:\n"
        s += '\n'.join(map(str, self._objets))
        return s

    def __contains__(self, p):
        r"""
        Returns True if the point p is in self.

        INPUT:

        - ``p`` - point in the space

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteSubset
            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: I = p & d3
            sage: vector((0,0,0)) in I
            True
        """
        return all(p in o for o in self._objets)

    def has_edge(self, p, s):
        r"""
        Returns whether it has the edge (p, s) where s-p is a canonical
        vector.

        INPUT:

        - ``p`` - point in the space
        - ``s`` - point in the space

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteSubset
            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: I = p & d3
            sage: I.has_edge(vector((0,0,0)),vector((0,0,1)))
            True
            sage: I.has_edge(vector((0,0,0)),vector((0,0,-1)))
            False

        TESTS::

            sage: from slabbe import DiscreteBox
            sage: F = lambda p: (2*p[0]+5*p[1]) % 7
            sage: edge_predicate = lambda p,s: F(p) < F(s)
            sage: D = DiscreteSubset(dimension=2, edge_predicate=edge_predicate)
            sage: b = DiscreteBox([-5,5],[-5,5])
            sage: I = D & b
            sage: all(I.has_edge(a,b) for a,b in I.edges_iterator())
            True
            sage: all(D.has_edge(a,b) for a,b in I.edges_iterator())
            True

        ::

            sage: from slabbe import ChristoffelGraph
            sage: C = ChristoffelGraph((2,5))
            sage: b = DiscreteBox([-5,5],[-5,5])
            sage: I = C & b
            sage: all(I.has_edge(a,b) for a,b in I.edges_iterator())
            True
            sage: all(C.has_edge(a,b) for a,b in I.edges_iterator())
            True

        """
        return all(o.has_edge(p,s) for o in self._objets)

    def __and__(self, other):
        r"""
        Return the intersection of self and other.

        INPUT:

        - ``other`` - Discrete object of dimension equal to s.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteSubset
            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: I = p & d3
            sage: I & p
            Intersection of the following objects:
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
            Subset of ZZ^3
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8

        ::

            sage: d3 = DiscreteSubset(dimension=3)
            sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
            sage: I = p & d3
            sage: p & I
            Intersection of the following objects:
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
            Subset of ZZ^3
            Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
        """
        if not isinstance(other, DiscreteSubset):
            raise TypeError("other(=%s) must be an instance of DiscreteSubset" % other)
        L = list(self._objets)
        if isinstance(other, Intersection):
            L.extend(other._objets)
        else:
            L.append(other)
        return Intersection(L)

    def an_element(self):
        r"""
        Returns an element in self.

        EXAMPLES::

            sage: from slabbe import DiscretePlane, DiscreteTube
            sage: P = DiscretePlane([4,6,7], 17, mu=0)
            sage: tube = DiscreteTube([-6.4, 6.4], [-5.2, 5.2])
            sage: I = tube & P
            sage: I.an_element()
            (0, 0, 0)
            sage: I.an_element() in I
            True

        TESTS::

            sage: P = DiscretePlane([4,6,7], 17, mu=0)
            sage: def contain(p): return 0 < P._v.dot_product(p) + P._mu <= P._omega
            sage: P._predicate = contain
            sage: tube = DiscreteTube([-6.4, 6.4], [-5.2, 5.2])
            sage: I = tube & P
            sage: I.an_element()
            (0, 0, 0) not in the plane
            trying similar points
            (0, 0, 1)

        """
        for o in self._objets:
            e = o.an_element()
            if e in self:
                return e
        else:
            raise ValueError("unable to find an element in this intersection")


class DiscreteBox(DiscreteSubset):
    r"""
    Cartesian product of intervals.

    INPUT:

    - ``*args`` - intervals, lists of size two : [min, max]

    EXAMPLES::

        sage: from slabbe import DiscreteBox
        sage: DiscreteBox([-5,5],[-5,5])
        Box: [-5, 5] x [-5, 5]

    ::

        sage: D = DiscreteBox([-3,3],[-3,3],[-3,3],[-3,3])
        sage: next(iter(D))
        (0, 0, 0, 0)

    TESTS::

        sage: d = DiscreteBox([-5,5], [-5,5], [-4,4])
        sage: next(d.edges_iterator())
        ((0, 0, 0), (1, 0, 0))

    """
    def __init__(self, *args):
        r"""
        Constructor.

        EXAMPLES::

            sage: from slabbe import DiscreteBox
            sage: b = DiscreteBox([2,10],[3,4])
            sage: b
            Box: [2, 10] x [3, 4]
            sage: b.roots()
            [(6, 4)]
        """
        self._intervals = args
        def predicate(p):
            return all(xmin <= x <= xmax for (x,(xmin,xmax)) in zip(p,self._intervals))
        dim = len(self._intervals)
        root = tuple(round((xmax+xmin)/2) for (xmin,xmax) in self._intervals)
        DiscreteSubset.__init__(self, dimension=dim,
                predicate=predicate, roots=[root])

    def __str__(self):
        r"""
        String representation.

        EXAMPLES::

            sage: from slabbe import DiscreteBox
            sage: str(DiscreteBox([2,10],[3,4]))
            '[2, 10] x [3, 4]'
            sage: str(DiscreteBox([-5,5], [-5,5], [-1,3]))
            '[-5, 5] x [-5, 5] x [-1, 3]'
        """
        L = ( "[%s, %s]" % (a, b) for (a,b) in self._intervals)
        return " x ".join(L)

    def _repr_(self):
        r"""
        String representation.

        EXAMPLES::

            sage: from slabbe import DiscreteBox
            sage: DiscreteBox([-5,5])
            Box: [-5, 5]
            sage: DiscreteBox([-5,5], [-5,5])
            Box: [-5, 5] x [-5, 5]
            sage: DiscreteBox([-5,5], [-5,5], [-4,4])
            Box: [-5, 5] x [-5, 5] x [-4, 4]
        """
        return "Box: %s" % self

    def clip(self, space=1):
        r"""
        Return a good clip rectangle for this box.

        INPUT:

        - ``space`` -- number (default: ``1``), inner space within the box

        EXAMPLES::

            sage: from slabbe import DiscreteBox
            sage: box = DiscreteBox([-6,6],[-6,6])
            sage: box
            Box: [-6, 6] x [-6, 6]
            sage: box.clip()
            [(-5, -5), (5, -5), (5, 5), (-5, 5), (-5, -5)]

        ::

            sage: box = DiscreteBox([-6,6],[-4,3])
            sage: box.clip()
            [(-5, -3), (5, -3), (5, 2), (-5, 2), (-5, -3)]
        """
        if self.dimension() != 2:
            raise NotImplementedError("implemented only for dimension (=%s) 2" % self.dimension())
        xmin,xmax = self._intervals[0]
        ymin,ymax = self._intervals[1]
        xmin = xmin + space
        xmax = xmax - space
        ymin = ymin + space
        ymax = ymax - space
        return [(xmin,ymin), (xmax,ymin), (xmax, ymax), (xmin,ymax),
                (xmin,ymin)]

class DiscreteTube(DiscreteSubset):
    r"""
    Discrete Tube (preimage of a box by a projection matrix)

    Subset of a discrete object such that its projection by a matrix is
    inside a certain box.

    INPUT:

    - ``*args`` - intervals, lists of size two : [min, max]
    - ``projmat`` - matrix (default: ``M3to2``), projection matrix

    EXAMPLES::

        sage: from slabbe import DiscreteTube
        sage: DiscreteTube([-5,5],[-5,5])
        DiscreteTube: Preimage of [-5, 5] x [-5, 5] by a 2 by 3 matrix

    ::

        sage: m = matrix(3,4,range(12))
        sage: DiscreteTube([2,10],[3,4],[6,7], projmat=m)
        DiscreteTube: Preimage of [2, 10] x [3, 4] x [6, 7] by a 3 by 4 matrix

    EXAMPLES::

        sage: from slabbe import DiscretePlane, DiscreteTube
        sage: p = DiscretePlane([1,pi,7], 1+pi+7, mu=0)
        sage: tube = DiscreteTube([-5,5],[-5,5])
        sage: I = p & tube
        sage: I
        Intersection of the following objects:
        Set of points x in ZZ^3 satisfying: 0 <= (1, pi, 7) . x + 0 < pi + 8
        DiscreteTube: Preimage of [-5, 5] x [-5, 5] by a 2 by 3 matrix
        sage: len(list(I))
        115
    """
    @options(projmat=M3to2)
    def __init__(self, *args, **kwds):
        r"""
        Constructor.

        Return the subset of self such that the projection by the matrix is
        inside a certain box.

        EXAMPLES::

            sage: from slabbe import DiscreteTube
            sage: DiscreteTube([2,10],[3,4])
            DiscreteTube: Preimage of [2, 10] x [3, 4] by a 2 by 3 matrix

        """
        self._box = DiscreteBox(*args)
        projmat = kwds['projmat']
        def predicate(p):
            return projmat * p in self._box
        root = projmat.pseudoinverse() * self._box.an_element()
        root = vector(map(round, root))
        DiscreteSubset.__init__(self, dimension=projmat.ncols(),
                predicate=predicate,roots=[root])
        self._projmat = projmat

    def _repr_(self):
        r"""
        String representation.

        EXAMPLES::

            sage: from slabbe import DiscreteTube
            sage: DiscreteTube([2,10],[3,4])
            DiscreteTube: Preimage of [2, 10] x [3, 4] by a 2 by 3 matrix
        """
        s = "DiscreteTube: Preimage of "
        s += str(self._box)
        s += " by a %s by %s matrix" % (self._projmat.nrows(), self._projmat.ncols())
        return s

    def clip(self, space=1):
        r"""
        Return a good clip rectangle for this box.

        INPUT:

        - ``space`` -- number (default: ``1``), inner space within the box

        EXAMPLES::

            sage: from slabbe import DiscreteTube
            sage: tube = DiscreteTube([-6,6],[-4,3])
            sage: tube.clip()
            [(-5, -3), (5, -3), (5, 2), (-5, 2), (-5, -3)]
        """
        return self._box.clip(space=space)




# -*- coding: utf-8 -*-
r"""
Simultaneous diophantine approximation

EXAMPLES:

The code gets between 1000x faster and 50000x faster compared to the same
function in Python code (diophantine_approximation.py)::

    sage: from slabbe.diophantine_approx_pyx import good_simultaneous_convergents_upto
    sage: from slabbe.diophantine_approx import _best_simultaneous_convergents_upto
    sage: good_simultaneous_convergents_upto([e,pi], 203)     # 493 µs
    [678, 19090, 19768]
    sage: _best_simultaneous_convergents_upto([e,pi], 203)         # 905 ms
    ((1843, 2130, 678), 203.239442934072)
    sage: 905/493. * 1000
    1835.69979716024

::

    sage: good_simultaneous_convergents_upto([e,pi], 204)    # 493 µs
    [19090, 19768, 41577]
    sage: _best_simultaneous_convergents_upto([e,pi], 204)         # 25s (not tested)
    ((51892, 59973, 19090), 266.17)
    sage: 25 / 493. * 1000000
    50709.9391480730

AUTHORS:

- Sébastien Labbé, October 19, 2016
"""
#*****************************************************************************
#       Copyright (C) 2016 Sebastien Labbe <slabqc@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************
from __future__ import absolute_import, print_function

from libc.math cimport floor

from cysignals.signals cimport sig_check   # ctrl-c interrupt block support
from cysignals.memory cimport check_allocarray, sig_free

def good_simultaneous_convergents_upto(v, double Q, int start=1, int step=1):
    r"""
    Return a list of potential best simultaneous diophantine approximation of
    vector ``v`` at distance ``1/Q``.

    It searches for all possibilities of denominators in the interval [1, Q^d]
    starting at start by step. Argument step is used for parallel
    computations purposes.

    INPUT:

    - ``v`` -- list of real numbers to approximate
    - ``Q`` -- real number, Q>1
    - ``start`` -- integer (default: ``1``), starting value to check
    - ``step`` -- integer (default: ``1``), step

    OUTPUT:

    - list of integers q such that entries of q*v are at most 1/Q from the
      integer lattice

    EXAMPLES::

        sage: from slabbe.diophantine_approx_pyx import good_simultaneous_convergents_upto
        sage: good_simultaneous_convergents_upto([e,pi], 2)
        [1, 2, 3, 4]
        sage: good_simultaneous_convergents_upto([e,pi], 4)
        [7, 14, 15]
        sage: good_simultaneous_convergents_upto([e,pi], 35)
        [7, 678, 685]
        sage: good_simultaneous_convergents_upto([e,pi], 36)
        [678, 685]
        sage: good_simultaneous_convergents_upto([e,pi], 203)
        [678, 19090, 19768]
        sage: good_simultaneous_convergents_upto([e,pi], 204)
        [19090, 19768, 41577]

    The previous results mean that these integers are good approximations::

        sage: v = [e,pi]
        sage: [(7 * a).n() for a in v]
        [19.0279727992133, 21.9911485751286]
        sage: [(678 * a).n() for a in v]
        [1842.99507969523, 2129.99981913388]
        sage: [(19090 * a).n() for a in v]
        [51892.0001052832, 59973.0037570292]

    Knowing the error of previous results, we can start the next computation at
    step 678::

        sage: min(1/abs(678*a - p) for a,p in zip([e,pi], (1843, 2130))).n()
        203.239442934072
        sage: good_simultaneous_convergents_upto([e,pi], 204, start=678)
        [19090, 19768, 41577]

    Use start and step for parallel computation purposes::

        sage: good_simultaneous_convergents_upto([e,pi], 2935, start=1)
        [432134, 1460669, 7961091, 8393225]
        sage: good_simultaneous_convergents_upto([e,pi], 2935, start=1, step=3)
        []
        sage: good_simultaneous_convergents_upto([e,pi], 2935, start=2, step=3)
        [432134, 1460669, 8393225]
        sage: good_simultaneous_convergents_upto([e,pi], 2935, start=3, step=3)
        [7961091]

    TESTS::

        sage: good_simultaneous_convergents_upto([1,e,pi], 1)
        Traceback (most recent call last):
        ...
        ValueError: argument Q(=1.0) must be > 1

    If the interval is too short, then noting is found::

        sage: good_simultaneous_convergents_upto([e,pi], 204, start=42000, step=2)
        []

    No solution was found when the q was `cdef int` instead of `cdef long`::

        sage: good_simultaneous_convergents_upto([e,pi], 136500, start=1)  # not tested (2min)
        [3111494861]

    This used to give a Value Error::

        sage: good_simultaneous_convergents_upto([e,pi], 102300, start=10^9, step=2) # not tested (1min)
        []
        sage: good_simultaneous_convergents_upto([e,pi], 102300, start=10^9+1, step=2) # not tested (1min)
        [3111494861, 3398281569]

    """
    if not Q > 1:
        raise ValueError("argument Q(={}) must be > 1".format(Q))
    cdef int d = len(v)
    cdef double Qinv = 1. / Q
    cdef double un_moins_Qinv = 1 - Qinv
    cdef double a
    cdef list L
    cdef double* vdouble = <double*>check_allocarray(d, sizeof(double))
    cdef int i
    cdef long q
    cdef long stop = int(floor(Q**d)) + 1
    for i in range(d):
        vdouble[i] = v[i]
    # for q in range(start, stop, step):
    # because of the use of step, the range is not translated into a C for-loop
    # http://stackoverflow.com/questions/21382180/cython-pure-c-loop-optimization
    L = []
    for q from start <= q < stop by step:
        sig_check() # Check for Keyboard interupt
        for i in range(d):
            a = q*vdouble[i]
            a = a - floor(a) 
            if not (a <= Qinv or un_moins_Qinv <= a):
                break
        else:
            L.append(q)
    sig_free(vdouble)
    return L

